"""Interface to flup and paste.httpserver

This module ties together the multi-process socket listening
capabilities of flup (see akara.thirdparty.preforkserver) with the
HTTP-to-WSGI capabilities of paste (see akara.thirdparty.httpserver).
It also handles the Akara extension modules.

Flup implements the Apache multi-processing module algorithm, which is
a non-threaded, pre-forking web server. The master process starts up
and kicks off a number of subprocesses, each listening to the same
socket. One of the subprocesses handles an incoming HTTP connection.

The master node keeps track of the status of each of these
subprocesses. If the load is too high it starts additional
subprocesses, up to a max. If the load is too low, it lowers the
number of processes down to a min. See the flup code for details.

When an HTTP request comes in, the flup-based AkaraPreforkServer in
the subprocess passes that off to and AkaraJob to handle the HTTP
connection. AkaraJob is a small adapter to ask paste to handle the
HTTP request. Paste knows how to convert the request into a WSGI
request, and then call AkaraWSGIDispatcher.wsgi_application.

AkaraWSGIDispatcher uses the registry to look up a WSGI handler for
the requested mount-point (the /first/word/in/the/path) and call it,
or report an error.

This module also contains code to read a set of Akara extension
modules. Each module is read and byte-compiled with an environment
which includes the special module variable "AKARA" which contains
configuration information for the module. The byte-code is exec'ed in
each HTTP listener subprocesses, which is also when module resource
registration occurs.

"""
import datetime
import os
import string
import sys
import time
import traceback
import urllib
from cStringIO import StringIO
import logging

from wsgiref.util import shift_path_info
from wsgiref.simple_server import WSGIRequestHandler

from akara import logger
from akara import registry

from akara.thirdparty import preforkserver, httpserver

access_logger = logging.getLogger("akara.access")

# AkaraPreforkServer creates and manages the subprocesses which are
# listening for HTTP requests. When a new connection request comes in
# it instanciates jobClass(sock, addr, *jobArgs) to process the
# connection.

# There is a minor complication here because the master node (which
# manages the subprocesses which are listening for HTTP requests) is
# the one which byte-compiles the extension modules during server
# start-up, as a sanity check that they make sense. The master process
# does not exec the extension modules. That is done in the child.

# Why wait? For one, it makes it hard for the extension modules to
# affect the master node. Core dumps from C extensions can't crash an
# Akara instance, slow memory leaks won't be a problem since flup
# automatically respawns a node after a number of requests, and so on.
# It's even possible to upgrade the extension modules and the modules
# it depends on then send a SIGHUP to the master to restart
# everything.

# The difficulty is that PreforkServer has no defined hook I can grab
# as a signal to exec the extension module bytecode. The jobClass is
# init'ed once for each request. I tried using a global variable to do
# the exec only when the first request comes in, but that slows down
# the first request (once for each number of spawned
# subprocesses).

# Instead, by inspection I found that self._child() is an internal
# method that I can use to sneak in my exec before letting flup's
# child mainloop run.

class AkaraPreforkServer(preforkserver.PreforkServer):
    def __init__(self, settings, config,
                 minSpare=1, maxSpare=5, maxChildren=50,
                 maxRequests=0):
        preforkserver.PreforkServer.__init__(self,
                                             minSpare=minSpare, maxSpare=maxSpare,
                                             maxChildren=maxChildren, maxRequests=maxRequests,
                                             jobClass=AkaraJob,
                                             jobArgs=(settings, config))
        self.config = config
    def _child(self, sock, parent):
        _init_modules(self.config)
        preforkserver.PreforkServer._child(self, sock, parent)


# Once the flup PreforkServer has a request, it starts up an AkaraJob.
# I'll let paste's WSGIHandler do the work of converting the HTTP
# request to a WSGI request. I actually use my own AkaraWSGIHandler
# because I want to change a few settings.

# AkaraWSGIHandler's third parameter, which is an 

class AkaraJob(object):
    def __init__(self, sock, addr, settings, config):
        self._sock = sock
        self._addr = addr
        self.settings = settings  # parsed settings as a dict
        self.config = config      # a ConfigParser
    def run(self):
        self._sock.setblocking(1)
        logger.debug("Start request from address %r, local socket %r" %
                     (self._addr, self._sock.getsockname()))
        handler = AkaraWSGIDispatcher(self.settings, self.config)
        self.handler = AkaraWSGIHandler(self._sock, self._addr, handler)
        logger.debug("End request from address %r, local socket %r" %
                     (self._addr, self._sock.getsockname()))
        self._sock.close()


# Override a few of the default settings
class AkaraWSGIHandler(httpserver.WSGIHandler):
    sys_version = None  # Disable including the Python version number
    server_version = "Akara/2.0"  # Declare that we are an Akara server
    protocol_version = "HTTP/1.1" # Support (for the most part) HTTP/1.1 semantics

    # Suppress access log reporting from BaseHTTPServer.py
    def log_request(self, code='-', size='-'):
        pass

# This is the the top-level WSGI dispatcher between paste.httpserver
# and Akara proper. It only understand how to get the first part of
# the path (called the "mount_point") and get the associated handler
# from the registry.

ERROR_DOCUMENT_TEMPLATE = """<?xml version="1.0" encoding="ISO-8859-1"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
  <title>%(reason)s</title>
</head>
<body>
  <h1>%(reason)s</h1>
  <p>
  %(message)s
  </p>
  <h2>Error %(code)s</h2>
</body>
</html>
"""

def _send_error(start_response, code, exc_info=None):
    reason, message = WSGIRequestHandler.responses[code]
    start_response("%d %s" % (code, reason), [("Content-Type", "text/html")],
                   exc_info=exc_info)
    return ERROR_DOCUMENT_TEMPLATE % dict(code = code,
                                          reason = reason,
                                          message = message)

# Output will look like Apache's "combined log format".
#    Here's an example based on the Apache documentation (it should be on a single line)
# 127.0.0.1 - frank [10/Oct/2000:13:55:36 -0700] "GET /apache_pb.gif HTTP/1.0"
# 200 2326 "http://www.example.com/start.html" "Mozilla/4.08 [en] (Win98; I ;Nav)"

# This definition comes from paste.translogger. For certainty's sake:
# (c) 2005 Ian Bicking and contributors; written for Paste (http://pythonpaste.org)
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
ACCESS_LOG_MESSAGE = (
    '%(REMOTE_ADDR)s - %(REMOTE_USER)s [%(start_time)s] '
    '"%(REQUEST_METHOD)s %(REQUEST_URI)s %(HTTP_VERSION)s" '
    '%(status)s %(bytes)s "%(HTTP_REFERER)s" "%(HTTP_USER_AGENT)s"')

# This proved a lot more difficult than I thought it would be.
# I looked at the translogger solution, but I don't think it works
# across the change of timezones and I didn't want to use the '%b'
# time formatter because it is locale dependant.

def timetuple_to_datetime(t):
    return datetime.datetime(t.tm_year, t.tm_mon, t.tm_mday, t.tm_hour, t.tm_min, t.tm_sec)

_months = "XXX Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split()
def _get_time():
    now = time.localtime()
    utc_time = time.gmtime()

    tz_seconds = (timetuple_to_datetime(now) - timetuple_to_datetime(utc_time)).seconds
    # Round to the nearest minute
    tz_minutes = (tz_seconds + 30)//60
    tz_hour, tz_minute = divmod(tz_minutes, 60)

    # I've got the timezone component. The rest is easy
    return "%02d/%s/%d:%02d:%02d:%02d %+03d%02d" % (
        now.tm_year, _months[now.tm_mon], now.tm_mday,
        now.tm_hour, now.tm_min, now.tm_sec,
        tz_hour, tz_minute)

# Filter out empty fields, control characters, and space characters
_illegal = ("".join(chr(i) for i in range(33)) +   # control characters up to space (=ASCII 32)
            "".join(chr(i) for i in range(127, 256)) ) # high ASCII
_clean_table = string.maketrans(_illegal, "?" * len(_illegal))
def _clean(s):
    # Check for empty fields
    if not s:
        return "-"
    # Filter control characters. These can be used for
    # escape character attacks against the terminal.
    # Plus, spaces can mess up access log parsers.
    return s.translate(_clean_table)
    

class AkaraWSGIDispatcher(object):
    def __init__(self, settings, config):
        self.server_address = settings["server_address"]

    def wsgi_application(self, environ, start_response):
        # There's some sort of problem if the application
        # hasn't read any data. This can occur, for example,
        # when sending a POST to a GET service, returning
        # a 405 message.
        wsgi_input = environ["wsgi.input"]
        try:
            return self._wsgi_application(environ, start_response)
        finally:
            # change the "if 1" to "if 0" and run
            #   test_server.test_405_error_message_mega
            # You should get socket.error "Connection reset by peer" errors.
            if 1 and isinstance(wsgi_input, httpserver.LimitedLengthFile):
                # Consume something if nothing was consumed *and* work
                # around a bug where paste.httpserver allows negative lengths
                if (wsgi_input._consumed == 0 and wsgi_input.length > 0):
                    # This seems to work even if there's 10K of input.
                    wsgi_input.read(1)
            
    def _wsgi_application(self, environ, start_response):
        # Get information used for access logging
        request_uri = urllib.quote(environ.get("SCRIPT_NAME", "") +
                                   environ.get("PATH_INFO", ""))
        if environ.get("QUERY_STRING"):
            request_uri += "?" + environ["QUERY_STRING"]
        # Convert all HEAD requests to GET requests before sending
        # them through the stack. Process the result to conform to
        # requirements for HEAD then strip out the body. This
        # follows the Apache/mod_wsgi approach. See
        # http://blog.dscpl.com.au/2009/10/wsgi-issues-with-http-head-requests.html
        is_head_request = environ.get("REQUEST_METHOD") == "HEAD"
        if is_head_request:
            environ["REQUEST_METHOD"] = "GET"

        access_data = dict(start_time = _get_time(),
                           request_uri = request_uri,
                           # Will get the following two from start_response_
                           status=None, content_length="-")

        # Set up some middleware so I can capture the status and header
        # information used for access logging.
        def start_response_(status, headers, exc_info=None):
            access_data["status"] = status.split(" ", 1)[0]
            access_data["content_length"] = "-"
            content_length = None
            headers = list(headers)
            for i, (k, v) in enumerate(headers):
                s = k.lower()
                if s == "content-length":
                    access_data["content_length"] = v
                elif s == "allow":
                    # Append a HEAD if a GET is allowed.
                    # Can't simply test:  if "GET" in s
                    #  since the Allow might be for "GET2".
                    # Parse and test the fields.
                    terms = [term.strip() for term in v.split(",")]
                    if "GET" in terms:
                        terms.append("HEAD")
                        headers[i] = (k, ", ".join(terms))
                    
            
            # Forward things to the real start_response
            return start_response(status, headers, exc_info)

        # Get the handler for this mount point
        if "/" not in environ.get("PATH_INFO", "/"):
            # This happens with very ill-formed queries
            # Like when you use httplib directly and forget the leading '/'.
            return _send_error(start_response, 400)
        mount_point = shift_path_info(environ)

        # Call the handler, deal with any errors, do access logging
        try:
            try:
                service = registry.get_service(mount_point)
            except KeyError:
                # Not found. Report something semi-nice to the user
                return _send_error(start_response_, 404)
            try:
                result = service.handler(environ, start_response_)
                if is_head_request:
                    # successful HEAD requests MUST return an empty message-body
                    return []
                return result
            except Exception, err:
                exc_info = sys.exc_info()
                try:
                    f = StringIO()
                    traceback.print_exc(file=f)
                    logger.error("Uncaught exception from %r (%r)\n%s" %
                                 (mount_point, service.ident, f.getvalue()))
                    return _send_error(start_response_, 500, exc_info=exc_info)
                finally:
                    del exc_info
        finally:
            self.save_to_access_log(environ, access_data)


            
    def save_to_access_log(self, environ, access_data):
        fields = dict(REMOTE_ADDR = _clean(environ.get("REMOTE_ADDR")),
                      REMOTE_USER = _clean(environ.get("REMOTE_USER")),
                      start_time = access_data["start_time"],
                      REQUEST_METHOD = _clean(environ.get("REQUEST_METHOD")),
                      REQUEST_URI = _clean(access_data["request_uri"]),
                      HTTP_VERSION = environ.get("SERVER_PROTOCOL"),
                      status = access_data["status"],
                      bytes = access_data["content_length"],
                      HTTP_REFERER = _clean(environ.get("HTTP_REFERER")),
                      HTTP_USER_AGENT = _clean(environ.get("HTTP_USER_AGENT")),
                      )
        access_logger.debug(ACCESS_LOG_MESSAGE % fields)


###### Support extension modules

# This used to be in its own module but that module was so small and
# almost pointless so I moved it into here. It doesn't feel quite
# right to be part of multiprocess_http, but it's close enough.

# The master HTTP process uses this module to import the modules and
# convert them into byte code with the correct globals(). It does not
# exec the byte code. That's the job for the spawned-off HTTP listener
# classes.

def _init_modules(config):
    try:
        modules = config["MODULES"]
    except KeyError:
        logger.error("Akara config file is missing the 'MODULES' variable.\n"
                     "No extensions will be installed.")
        return
    for module_name in modules:
        # import the module
        try:
            __import__(module_name)
        except:
            logger.error(
"Unable to initialize module %r - skipping rest of module" % (module_name,),
                         exc_info = True)
    



# -*- coding: iso-8859-1 -*-
# 
"""
moin2cms.py (Akara demo)

Accesses a Moin wiki (via the moinrest module) to use as a source for a Web feed

Copyright 2009 Uche Ogbuji
This file is part of the open source Akara project,
provided under the Apache 2.0 license.
See the files LICENSE and NOTICE for details.
Project home, documentation, distributions: http://wiki.xml3k.org/Akara

@copyright: 2009 by Uche ogbuji <uche@ogbuji.net>
"""

from __future__ import with_statement
import os
import sys
import re
import pprint
import httplib
import urllib, urllib2
import datetime
import cgi
from string import Template
from cStringIO import StringIO
from functools import partial
from itertools import *
from contextlib import closing

# Requires Python 2.6 or http://code.google.com/p/json/
from amara.thirdparty import json
from dateutil.parser import parse as dateparse

import amara
from amara import bindery, _
from amara.namespaces import *
from amara.bindery.model import generate_metadata
from amara.bindery.model.examplotron import examplotron_model
from amara.writers.struct import *
from amara.bindery.html import parse as htmlparse
from amara.lib import U
from amara.lib.iri import split_fragment, relativize, absolutize
from amara.lib.date import timezone, UTC
from amara.bindery.util import dispatcher, node_handler, property_sequence_getter

from akara.util import copy_auth
from akara.util.moin import node, ORIG_BASE_HEADER, DOCBOOK_IMT, RDF_IMT, HTML_IMT, CMS_BASE, register_node_type
from akara.services import simple_service


CMS_BASE = "asdfasd" # Get from akara_init config? XXX

#
# This part is partly obsolete, and is used to handle the Web/CMS component.
# It needs a bit of update for the more general Moin/CMS framework
# FIXME: It should actually probably go in a different file
#

class webcms_node(node):
    '''
    Akara CMS node, a Moin wiki page in a lightly specialized format
    from which semi-structured information can be extracted
    '''
    NODES = {}
    #Processing priority
    PRIORITY = 0
    @staticmethod
    def factory(rest_uri, relative, outputdir):
        req = urllib2.Request(rest_uri, headers={'Accept': DOCBOOK_IMT})
        resp = urllib2.urlopen(req)
        doc = bindery.parse(resp, standalone=True, model=MOIN_DOCBOOK_MODEL)
        original_wiki_base = dict(resp.info())[ORIG_BASE_HEADER]
        #self.original_wiki_base = dict(resp.info())[ORIG_BASE_HEADER]
        #amara.xml_print(self.content_cache)
        output = os.path.join(outputdir, relative)
        parent_dir = os.path.split(output)[0]
        try:
            os.makedirs(parent_dir)
        except OSError:
            pass
        metadata, first_id = metadata_dict(generate_metadata(doc))
        metadata = metadata[first_id]
        akara_type = first_item(first_item(metadata[u'ak-type']))
        #import sys; print >> sys.stderr, 'GRIPPO', akara_type.xml_value
        cls = node.NODES[akara_type.xml_value]
        instance = cls(rest_uri, relative, outputdir, cache=(doc, metadata, original_wiki_base))
        return instance

    def __init__(self, rest_uri, relative, outputdir, cache=None):
        '''
        rest_uri - the full URI to the Moin/REST wrapper for this page
        relative - the URI of this page relative to the Wiki base
        '''
        self.relative = relative
        self.rest_uri = rest_uri
        self.output = os.path.join(outputdir, relative)
        self.outputdir = outputdir
        self.cache = cache#(doc, metadata)
        return

    def load(self):
        return

    def up_to_date(self, force_update=False):
        '''
        Checks whether there needs to be an update of the CMS output file or folder
        '''
        #By default just always update
        return False
        if force_update:
            self.load()
        doc, metadata, original_wiki_base = self.cache
        entrydate = dateparse(unicode(doc.article.articleinfo.revhistory.revision.date))
        if entrydate.tzinfo == None: entrydate = entrydate.replace(tzinfo=UTC)
        if not os.access(self.output, os.R_OK):
            return False
        try:
            lastrev = dateparse(unicode(bindery.parse(self.output).entry.updated))
        except amara.ReaderError:
            return False
        if lastrev.tzinfo == None: lastrev = lastrev.replace(tzinfo=UTC)
        if (entrydate == lastrev):
            #print >> sys.stderr, 'Not updated.  Skipped...'
            return False
        return True


class folder(webcms_node):
    AKARA_TYPE = CMS_BASE + u'/folder'
    PRIORITY = 1000
    def render(self):
        #Copy attachments to dir
        req = urllib2.Request(self.rest_uri, headers={'Accept': ATTACHMENTS_IMT})
        resp = urllib2.urlopen(req)
        doc = bindery.parse(resp, model=ATTACHMENTS_MODEL)
        for attachment in (doc.attachments.attachment or ()):
            print attachment
        return

node.NODES[folder.AKARA_TYPE] = folder


class page(webcms_node):
    AKARA_TYPE = CMS_BASE + u'/page'
    def up_to_date(self, force_update=False):
        '''
        Checks whether there needs to be an update of the CMS output file or folder
        '''
        if force_update:
            self.load()
        doc, metadata, original_wiki_base = self.cache
        pagedate = dateparse(unicode(doc.article.articleinfo.revhistory.revision.date))
        #Note from the Moin FAQ: http://moinmo.in/MoinMoinQuestions/UsingTheWiki
        #"Moin internally only uses UTC, but calculates your local time according to your UserPreferences setting on page view. If you set your timezone offset to 0, you get UTC."
        #Check the behavior if using the Moin REST wrapper with user auth where that user's prefs specify TZ
        if pagedate.tzinfo == None: pagedate = pagedate.replace(tzinfo=UTC)
        if not os.access(self.output, os.R_OK):
            return False
        lastrev = datetime.datetime.utcfromtimestamp(os.stat(self.output)[stat.ST_MTIME])
        #try:
        #    published_doc = bindery.parse(self.output)
        #    datestr = first_item([ m for m in published_doc.html.head.meta if m.name==u'updated']).content
        #    lastrev = dateparse(datestr)
        #except amara.ReaderError:
        #    return False
        if lastrev.tzinfo == None: lastrev = lastrev.replace(tzinfo=UTC)
        if (lastrev > pagedate):
            return True
        return False

    def render(self):
        '''
        The typical approach is along the lines of "Style-free XSLT Style Sheets"
        * http://www.xml.com/pub/a/2000/07/26/xslt/xsltstyle.html
        * http://www.cocooncenter.org/articles/stylefree.html
        But using div/@id rather than custome elements
        '''
        doc, metadata, original_wiki_base = self.cache
        self.content = content_handlers(original_wiki_base)
        #metadata = doc.article.xml_model.generate_metadata(doc)
        #import pprint
        #pprint.pprint(resources)
        '''
         akara:type:: [[http://purl.org/dc/org/xml3k/akara/cms/folder|folder]]
         title:: A page
         template:: http://wiki.example.com/Site;attachment=foo.xslt ##Just XSLT for now.  Plan to support other templating systems soon
         link:: [[http://example.org|]] rel=...
         meta:: dc:Creator value=Uche Ogbuji
         script:: `...` ##preferably they'd only use linked scripts: [[myscript...]]
        '''
        
        page_id = doc.article.xml_nodeid
        header = doc.article.glosslist[0]
        #node_type = first_item(header.xml_select(u'glossentry[glossterm = "akara:type"]/glossdef'))
        template = unicode(first_item(header.xml_select(u'glossentry[glossterm = "template"]/glossdef'))).strip()
        template = os.path.join(self.outputdir, template)
        title = first_item(header.xml_select(u'glossentry[glossterm = "title"]/glossdef'))
        #title = resources[articleid]['title']
        #sections = dict([ (unicode(s.title), s) for s in page.article.section ])
        #print sections
         # if unicode(g.glossterm) == u'page:header' ]
        #authors = [ a
        #    for a in page.article.section.glosslist.glossentry
        #    if unicode(a.glossterm) == u'entry:authors'
        #]
        #title = article.xml_select(u'section[@title = ]')

        #revdate = dateparse(unicode(page.article.articleinfo.revhistory.revision.date))
        #if revdate.tzinfo == None: revdate = revdate.replace(tzinfo=UTC)
        
        #Create ouput file
        #print >> sys.stderr, 'Writing to ', self.output
        buf = StringIO()
        w = structwriter(indent=u"yes", stream=buf)
        w.feed(
        ROOT(
            E((XHTML_NAMESPACE, u'html'), {(XML_NAMESPACE, u'xml:lang'): u'en'},
                E(u'head',
                    E(u'title', title),
                    E(u'meta', {u'content': unicode(first_item(metadata[u'ak-updated'])), u'name': u'updated'}),
                    #E(u'link', {u'href': unicode(uri), u'rel': u'alternate', u'title': u"Permalink"}),
                ),
                E(u'body',
                    (self.content.dispatch(s) for s in doc.article.section)
                ),
            ),
        ))
        with open(self.output, 'w') as output:
            #text = f.read().rstrip()
            #print buf.getvalue()
            transform(buf.getvalue(), template, output=output)
        return

    def meta(self):
        #Create ouput file
        doc = bindery.parse(source, model=AK_DOCBOOK_MODEL)

node.NODES[page.AKARA_TYPE] = page
#AKARA_TYPES = [page, folder]


class content_handlers(dispatcher):
    def __init__(self, orig_wikibase):
        dispatcher.__init__(self)
        self.orig_wikibase = orig_wikibase
        return

    @node_handler(u'article/section', priority=10)
    def top_section(self, node):
        yield E((XHTML_NAMESPACE, u'div'), {u'id': node.title},
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'section')
    def section(self, node):
        depth = node.xml_select(u'count(ancestor::section)')
        yield E((XHTML_NAMESPACE, u'h%i'%depth), unicode(node.title))
        for node in chain(*imap(self.dispatch, node.xml_children)):
            yield node

    @node_handler(u'section/title')
    def section_title(self, node):
        #Ignore this node
        raise StopIteration

    @node_handler(u'para')
    def para(self, node):
        #print 'content_handlers.para'
        yield E((XHTML_NAMESPACE, u'p'),
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'code')
    def code(self, node):
        yield E((XHTML_NAMESPACE, u'code'),
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'emphasis')
    def emphasis(self, node):
        ename = u'strong' if node.xml_attributes.get((None, u'role')) == u'strong' else u'em'
        yield E((XHTML_NAMESPACE, ename),
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'screen')
    def screen(self, node):
        yield E((XHTML_NAMESPACE, u'pre'),
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'ulink')
    def a(self, node):
        '''
        [[Some_page]] -> @url == $WIKIBASE/Some_page
        [[Some_page/Child]] -> @url == $WIKIBASE/Some_page/Child
        [[http://moinmo.in/]] -> @url == http://moinmo.in/
        '''
        url = node.url
        if url.startswith(self.orig_wikibase):
            url = url[len(self.orig_wikibase):]
        yield E((XHTML_NAMESPACE, u'a'), {u'href': url},
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'inlinemediaobject')
    def img(self, node):
        '''
        {{http://static.moinmo.in/logos/moinmoin.png}} -> img/@src=...
        '''
        url = node.imageobject.imagedata.fileref
        if url.startswith(self.orig_wikibase):
            url = url[len(self.orig_wikibase):]
        yield E((XHTML_NAMESPACE, u'img'), {u'src': url},
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'itemizedlist')
    def ul(self, node):
        '''
        * foo
        '''
        yield E((XHTML_NAMESPACE, u'ul'),
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'listitem')
    def li(self, node):
        yield E((XHTML_NAMESPACE, u'li'),
            chain(*imap(self.dispatch, [ grandchild for grandchild in node.para.xml_children ]))
        )

    #@node_handler(u'*', priority=-1)
    #def etc(self, node):


def moincms(wikibase, outputdir, pattern):
    if pattern: pattern = re.compile(pattern)
    #print (wikibase, outputdir, rewrite)
    req = urllib2.Request(wikibase, headers={'Accept': RDF_IMT})
    resp = urllib2.urlopen(req)
    original_wiki_base = dict(resp.info())[ORIG_BASE_HEADER]
    feed = bindery.parse(resp)
    process_list = []
    for item in feed.RDF.channel.items.Seq.li:
        uri = split_fragment(item.resource)[0]
        #print >> sys.stderr, (uri, str(item.resource), split_fragment(item.resource))
        #Deal with the wrapped URI
        if original_wiki_base:
            #print >> sys.stderr, (uri, original_wiki_base.rstrip('/')+'/')
            relative = relativize(uri, original_wiki_base.rstrip('/')+'/').lstrip('/')
            uri = absolutize(relative, wikibase)
        #print >> sys.stderr, (uri, relative)
        if pattern and not pattern.match(relative):
            continue
        n = node.factory(uri, relative, outputdir)
        if n.up_to_date():
            pass
            #print >> sys.stderr, 'Up to date.  Skipped...'
        else:
            process_list.append(n)
            
    #Process nodes needing update according to priority
    for n in sorted(process_list, key=attrgetter('PRIORITY'), reverse=True):
        #print >> sys.stderr, "processing ", n.rest_uri
        n.render()
    return

#Ideas borrowed from
# http://www.artima.com/forums/flat.jsp?forum=106&thread=4829

import sys
import SocketServer
from wsgiref import simple_server


def command_line_prep():
    from optparse import OptionParser
    usage = "%prog [options] wikibase outputdir"
    parser = OptionParser(usage=usage)
    parser.add_option("-p", "--pattern",
                      action="store", type="string", dest="pattern",
                      help="limit the pages treated as Atom entries to those matching this pattern")
    return parser


def main(argv=None):
    #But with better integration of entry points
    if argv is None:
        argv = sys.argv
    # By default, optparse usage errors are terminated by SystemExit
    try:
        optparser = command_line_prep()
        options, args = optparser.parse_args(argv[1:])
        # Process mandatory arguments with IndexError try...except blocks
        try:
            wikibase = args[0]
            try:
                outputdir = args[1]
            except IndexError:
                optparser.error("Missing output directory")
        except IndexError:
            optparser.error("Missing Wiki base URL")
    except SystemExit, status:
        return status

    # Perform additional setup work here before dispatching to run()
    # Detectable errors encountered here should be handled and a status
    # code of 1 should be returned. Note, this would be the default code
    # for a SystemExit exception with a string message.
    pattern = options.pattern and options.pattern.decode('utf-8')

    moincms(wikibase, outputdir, pattern)
    return


#aname = partial(property_sequence_getter, u"name")
#aemail = partial(property_sequence_getter, u"email")
#auri = partial(property_sequence_getter, u"uri")

UNSUPPORTED_IN_FILENAME = re.compile('\W')
#SOURCE = AKARA_MODULE_CONFIG['source-wiki-root']
#POST_TO = AKARA_MODULE_CONFIG['post-to']

class atom_entry(node):
    AKARA_TYPE = CMS_BASE + u'/atom-entry'
    OUTPUTPATTERN = None
    def __init__(self, rest_uri, relative, outputdir, cache=None):
        node.__init__(self, rest_uri, relative, outputdir, cache)
        self.relative = relative
        return

    def up_to_date(self, force_update=False):
        '''
        Checks whether there needs to be an update of the CMS output file or folder
        '''
        doc, metadata, original_wiki_base = self.cache
        entrydate = dateparse(unicode(doc.article.articleinfo.revhistory.revision.date))
        if entrydate.tzinfo == None: entrydate = entrydate.replace(tzinfo=UTC)
        output = os.path.join(outputdir, self.OUTPUTPATTERN%pathsegment(relative))
        if os.access(output, os.R_OK):
            lastrev = dateparse(unicode(bindery.parse(output).entry.updated))
            if lastrev.tzinfo == None: lastrev = lastrev.replace(tzinfo=UTC)
            if (entrydate == lastrev):
                print >> sys.stderr, 'Not updated.  Skipped...'
                # continue
                return

        if force_update:
            self.load()

        pagedate = dateparse(unicode(doc.article.articleinfo.revhistory.revision.date))
        #Note from the Moin FAQ: http://moinmo.in/MoinMoinQuestions/UsingTheWiki
        #"Moin internally only uses UTC, but calculates your local time according to your UserPreferences setting on page view. If you set your timezone offset to 0, you get UTC."
        #Check the behavior if using the Moin REST wrapper with user auth where that user's prefs specify TZ
        if pagedate.tzinfo == None: pagedate = pagedate.replace(tzinfo=UTC)
        if not os.access(self.output, os.R_OK):
            return False
        lastrev = datetime.datetime.utcfromtimestamp(os.stat(self.output)[stat.ST_MTIME])
        #try:
        #    published_doc = bindery.parse(self.output)
        #    datestr = first_item([ m for m in published_doc.html.head.meta if m.name==u'updated']).content
        #    lastrev = dateparse(datestr)
        #except amara.ReaderError:
        #    return False
        if lastrev.tzinfo == None: lastrev = lastrev.replace(tzinfo=UTC)
        if (lastrev > pagedate):
            return True
        return False

    def render(self):
        '''
        The typical approach is along the lines of "Style-free XSLT Style Sheets"
        * http://www.xml.com/pub/a/2000/07/26/xslt/xsltstyle.html
        * http://www.cocooncenter.org/articles/stylefree.html
        But using div/@id rather than custome elements
        '''
        doc, metadata, original_wiki_base = self.cache
        self.content = content_handlers(original_wiki_base)
        #metadata = doc.article.xml_model.generate_metadata(doc)
        #import pprint
        #pprint.pprint(resources)
        '''
         akara:type:: [[http://purl.org/xml3k/akara/xmlmodel/cms/folder|folder]]
         title:: A page
         template:: http://wiki.example.com/Site;attachment=foo.xslt ##Just XSLT for now.  Plan to support other templating systems soon
         link:: [[http://example.org|]] rel=...
         meta:: dc:Creator value=Uche Ogbuji
         script:: `...` ##preferably they'd only use linked scripts: [[myscript...]]
        '''
        
        page_id = doc.article.xml_nodeid
        header = doc.article.glosslist[0]
        #node_type = first_item(header.xml_select(u'glossentry[glossterm = "akara:type"]/glossdef'))
        template = unicode(first_item(header.xml_select(u'glossentry[glossterm = "template"]/glossdef'))).strip()
        template = os.path.join(self.outputdir, template)
        title = first_item(header.xml_select(u'glossentry[glossterm = "title"]/glossdef'))
        #title = resources[articleid]['title']
        #sections = dict([ (unicode(s.title), s) for s in page.article.section ])
        #print sections
         # if unicode(g.glossterm) == u'page:header' ]
        #authors = [ a
        #    for a in page.article.section.glosslist.glossentry
        #    if unicode(a.glossterm) == u'entry:authors'
        #]
        #title = article.xml_select(u'section[@title = ]')

        #revdate = dateparse(unicode(page.article.articleinfo.revhistory.revision.date))
        #if revdate.tzinfo == None: revdate = revdate.replace(tzinfo=UTC)
        
        #Create ouput file
        print >> sys.stderr, 'Writing to ', self.output
        buf = StringIO()
        w = structwriter(indent=u"yes", stream=buf)
        w.feed(
        ROOT(
            E((XHTML_NAMESPACE, u'html'), {(XML_NAMESPACE, u'xml:lang'): u'en'},
                E(u'head',
                    E(u'title', title),
                    E(u'meta', {u'content': U(metadata[u'ak-updated']), u'name': u'updated'}),
                    #E(u'link', {u'href': unicode(uri), u'rel': u'alternate', u'title': u"Permalink"}),
                ),
                E(u'body',
                    (self.content.dispatch(s) for s in doc.article.section)
                ),
            ),
        ))
        with open(self.output, 'w') as output:
            #text = f.read().rstrip()
            #print buf.getvalue()
            transform(buf.getvalue(), template, output=output)
        return

register_node_type(page.AKARA_TYPE, page)


class freemix(node):
    AKARA_TYPE = 'http://purl.org/dc/gov/loc/recollection/collection'
    def __init__(self, rest_uri, opener):
        self.rest_uri = rest_uri
        self.opener = opener
        #from node.factory
        req = urllib2.Request(rest_uri, headers={'Accept': DOCBOOK_IMT})
        print >> sys.stderr, 'rest_uri: ', rest_uri
        with closing(opener.open(req)) as resp:
            doc = bindery.parse(resp, standalone=True, model=MOIN_DOCBOOK_MODEL)
            original_wiki_base = dict(resp.info())[ORIG_BASE_HEADER]
            #self.original_wiki_base = dict(resp.info())[ORIG_BASE_HEADER]
            #amara.xml_print(self.content_cache)
        metadata = metadata_dict(generate_metadata(doc))
        self.cache=(doc, metadata, original_wiki_base)
        return

    def up_to_date(self, force_update=False):
        '''
        Checks whether there needs to be an update of the output
        '''
        return False

    def render(self):
        '''
        '''
        doc, metadata, original_wiki_base = self.cache
        #metadata = doc.article.xml_model.generate_metadata(doc)
        #import pprint
        #pprint.pprint(resources)
        #amara.xml_print(doc, stream=sys.stderr, indent=True)
        header = doc.article.glosslist[0]
        freemix_info = {
            'id': self.rest_uri,
            'label': self.rest_uri,
            'title': doc.article.xml_select(u'string(section[title = "collection:title"]/para)'),
            'date-created': header.xml_select(u'string(glossentry[glossterm = "date-created"]/glossdef)'),
            'description': doc.article.xml_select(u'string(section[title = "collection:description"]/para)'),
            'link': header.xml_select(u'string(glossentry[glossterm = "link"]/glossdef//ulink/@url)'),
            'original_site': doc.article.xml_select(u'string(section[title = "collection:original site"]/para)'),
            'organization': doc.article.xml_select(u'string(section[title = "collection:organization"]/para)'),
            'restrictions': doc.article.xml_select(u'string(section[title = "collection:restrictions"]/para)'),
            'content': doc.article.xml_select(u'string(section[title = "collection:content"]/para)'),
            'thumbnail': header.xml_select(u'string(glossentry[glossterm = "thumbnail"]/glossdef//ulink/@url)'),
            'tags': [ unicode(tag).strip() for tag in doc.article.xml_select(u'section[title = "collection:tags"]//para')],
        }
        #print >> sys.stderr, 'FINFO ', freemix_info
        return freemix_info

    def meta(self):
        #Create ouput file
        doc = bindery.parse(source, model=AK_DOCBOOK_MODEL)

node.NODES[page.AKARA_TYPE] = page
#AKARA_TYPES = [page, folder]
#print >> sys.stderr, 'Writing to ', POST_TO

SELF = AKARA.module_config.get('self', 'http://localhost:8880/')
REST_WIKI_BASE = AKARA.module_config.get('rest_wiki_base', 'http://localhost:8880/moin/loc/')


def wrapped_uri(original_wiki_base, link):
    abs_link = absolutize(link, original_wiki_base)
    #print >> sys.stderr, 'abs_link: ', abs_link
    rel_link = relativize(abs_link, original_wiki_base)
    #print >> sys.stderr, 'rel_link: ', rel_link
    rest_uri = absolutize(rel_link, REST_WIKI_BASE)
    #print >> sys.stderr, 'rest_uri: ', rest_uri
    return rest_uri

WIKI_REQUIRED = _("The 'wiki' query parameter is mandatory.")
PATTERN_REQUIRED = _("The 'pattern' query parameter is mandatory.")

DEFAULT_TRANSFORM = AKARA.module_config.get('default_transform')
#print DEFAULT_TRANSFORM

SERVICE_ID = 'http://purl.org/akara/services/demo/moincms.execute'
@simple_service('POST', SERVICE_ID, 'moincms.execute')
def execute(top=None):
    '''
    Sample request:
    curl -F "pattern=wiki/path" -F "wiki=http://localhost:8880/moin/foo/" "http://localhost:8880/moincms.execute"
    '''
    #
    #wikibase_len = len(rewrite)
    body = StringIO(body)
    form = cgi.FieldStorage(fp=body, environ=WSGI_ENVIRON)
    #for k in form:
    #    print >> sys.stderr, (k, form[k][:100])
    wiki = form.getvalue('wiki')
    assert_not_equal(wiki, None, msg=WIKI_REQUIRED)
    pattern = form.getvalue('pattern')
    assert_not_equal(pattern, None, msg=PATTERN_REQUIRED)
    pattern = re.compile(pattern)
    
    handler = copy_auth(WSGI_ENVIRON, top)
    opener = urllib2.build_opener(handler) if handler else urllib2.build_opener()
    req = urllib2.Request(wiki, headers={'Accept': RDF_IMT})
    with closing(opener.open(req)) as resp:
        feed = bindery.parse(resp)
        original_wiki_base = dict(resp.info())[ORIG_BASE_HEADER]

    for item in feed.RDF.channel.items.Seq.li:
        uri = split_fragment(item.resource)[0]
        relative = uri[len(original_wiki_base):]
        print >> sys.stderr, uri, relative
        if pattern and not pattern.match(relative):
            continue
        if rewrite:
            uri = uri.replace(rewrite, wikibase)
        req = urllib2.Request(uri, headers={'Accept': DOCBOOK_IMT})
        with closing(urllib2.urlopen(req)) as resp:
            page = bindery.parse(resp)

        print >> sys.stderr, 'Writing to ', output
        with open(output, 'w') as output:
            handle_page(uri, page, outputdir, relative, output)


        doc = htmlparse(resp)

    #print (wikibase, outputdir, rewrite)
    with closing(urllib2.urlopen(req)) as resp:
        raise NotImplementedError
    return



    #wikibase, outputdir, rewrite, pattern
    #wikibase_len = len(rewrite)
    items = []
    for navchild in doc.xml_select(u'//*[@class="navigation"]//@href'):
        link = navchild.xml_value
        #print >> sys.stderr, 'LINK:', link
        #uri = split_fragment(item.resource)[0]
        #relative = uri[wikibase_len:]
        #print >> sys.stderr, uri, relative
        #if rewrite:
        #    uri = uri.replace(rewrite, wikibase)
        rest_uri = wrapped_uri(original_wiki_base, link)
        #print >> sys.stderr, 'rest uri:', rest_uri
        items.append(freemix(rest_uri, opener).render())
    return json.dumps({'items': items}, indent=4)




# -*- encoding: utf-8 -*-
'''
A tool for convenient viewing of log files
'''

#See also: Beazley's approach: http://www.dabeaz.com/generators/Generators.pdf (slide 61 et seq)

import sys, time, re
import urllib2
import httplib
import datetime
from itertools import *

# Requires Python 2.6 or http://code.google.com/p/json/
from amara.thirdparty import json

#from amara.tools.atomtools import feed
from amara.tools import rdfascrape

from akara.services import simple_service

#def rdfa2json(url=None):
#Support POST body as well

LOGLINE_PAT = re.compile(r'(?P<origin>\d+\.\d+\.\d+\.\d+) '
+r'(?P<identd>-|\w*) (?P<auth>-|\w*) '
+r'\[(?P<date>[^\[\]:]+):(?P<time>\d+:\d+:\d+) (?P<tz>[\-\+]?\d\d\d\d)\] '
+r'(?P<request>"[^"]*") (?P<status>\d+) (?P<bytes>-|\d+) (?P<referrer>"[^"]*") (?P<client>".*")\s*\Z')

#For logs with destination host (at the beginning)
#LOGLINE_PAT = re.compile(r'(?P<targetserver>\S+) (?P<origin>\d+\.\d+\.\d+\.\d+) '
#+r'(?P<identd>-|\w*) (?P<auth>-|\w*) '
#+r'\[(?P<date>[^\[\]:]+):(?P<time>\d+:\d+:\d+) (?P<tz>[\-\+]?\d\d\d\d)\] '
#+r'(?P<request>"[^"]*") (?P<status>\d+) (?P<bytes>-|\d+) (?P<referrer>"[^"]*") (?P<client>".*")\s*\Z')

#For logs where referrer and UA may not be included
#LOGLINE_PAT = re.compile(r'(\d+\.\d+\.\d+\.\d+) (-|\w*) (-|\w*) '
#+r'\[([^\[\]:]+):(\d+:\d+:\d+) -(\d\d\d\d)\] '
#+r'("[^"]*") (\d+) (-|\d+) ("[^"]*")? (".*")?\s*\Z')

# This regular expresion is the heart of the code.
# Python uses Perl regex, so it should be readily portable
# The r'' string form is just a convenience so you don't have to escape backslashes
COMBINED_LOGLINE_PAT = re.compile(
  r'(?P<origin>[^\s]+) '
+ r'(?P<identd>-|\w*) (?P<auth>-|\w*) '
+ r'\[(?P<ts>(?P<date>[^\[\]:]+):(?P<time>\d+:\d+:\d+)) (?P<tz>[\-\+]?\d\d\d\d)\] '
+ r'"(?P<method>\w+) (?P<path>[\S]+) (?P<protocol>[^"]+)" (?P<status>\d+) (?P<bytes>-|\d+)'
+ r'( (?P<referrer>"[^"]*")( (?P<client>"[^"]*")( (?P<cookie>"[^"]*"))?)?)?\s*\Z'
)

# Patterns in the client field for sniffing out bots
BOT_TRACES = [
    (re.compile(r".*http://help\.yahoo\.com/help/us/ysearch/slurp.*"),
        "Yahoo robot"),
    (re.compile(r".*\+http://www\.google\.com/bot\.html.*"),
        "Google robot"),
    (re.compile(r".*\+http://about\.ask\.com/en/docs/about/webmasters.shtml.*"),
        "Ask Jeeves/Teoma robot"),
    (re.compile(r".*\+http://search\.msn\.com\/msnbot\.htm.*"),
        "MSN robot"),
    (re.compile(r".*http://www\.entireweb\.com/about/search_tech/speedy_spider/.*"),
        "Speedy Spider"),
    (re.compile(r".*\+http://www\.baidu\.com/search/spider_jp\.html.*"),
        "Baidu spider"),
    (re.compile(r".*\+http://www\.gigablast\.com/spider\.html.*"),
        "Gigabot robot"),
]

# Apache's date/time format is very messy, so dealing with it is messy
# This class provides support for managing timezones in the Apache time field
# Reuses some code from: http://seehuhn.de/blog/52
class timezone(datetime.tzinfo):
    def __init__(self, name="+0000"):
        self.name = name
        seconds = int(name[:-2])*3600+int(name[-2:])*60
        self.offset = datetime.timedelta(seconds=seconds)

    def utcoffset(self, dt):
        return self.offset

    def dst(self, dt):
        return timedelta(0)

    def tzname(self, dt):
        return self.name


def parse_apache_date(date_str, tz_str):
    '''
    Parse the timestamp from the Apache log file, and return a datetime object
    '''
    tt = time.strptime(date_str, "%d/%b/%Y:%H:%M:%S")
    tt = tt[:6] + (0, timezone(tz_str))
    return datetime.datetime(*tt)


def bot_check(match_info):
    '''
    Return True if the matched line looks like a robot
    '''
    for pat, botname in BOT_TRACES:
        if pat.match(match_info.group('client')):
            return True
            break
    return False


SERVICE_ID = 'http://purl.org/akara/services/demo/wwwlog.json'
@simple_service('POST', SERVICE_ID, 'akara.wwwlog.json', 'application/json')
def wwwlog2json(body, ctype, maxrecords=None, nobots=False):
    '''
    Convert Apache log info to Exhibit JSON
    (see: http://www.ibm.com/developerworks/web/library/wa-realweb6/ )

    Sample request:
    * curl --request POST --data-binary "@access.log" --header "Content-Type: text/plain" "http://localhost:8880/akara.wwwlog.json"
    '''
    if maxrecords:
        maxrecords = int(maxrecords)
    entries = []
    for count, line in enumerate(body.splitlines()):
        if maxrecords and count >= maxrecords:
            break
        match_info = COMBINED_LOGLINE_PAT.match(line)
        if not match_info:
            print >> sys.stderr, "Unable to parse log line: ", line
            continue
        if nobots and bot_check(match_info):
            continue
        entry = {}
        timestamp = parse_apache_date(match_info.group('ts'), match_info.group('tz'))
        timestamp_str = timestamp.isoformat()

        # To make Exhibit happy, set id and label fields that give some information
        # about the entry, but are unique across all entries (ensured by appending count)
        entry['id'] = match_info.group('origin') + ':' + timestamp_str + ':' + str(count)
        entry['label'] = entry['id']
        entry['origin'] = match_info.group('origin')
        entry['timestamp'] = timestamp_str
        entry['path'] = match_info.group('path')
        entry['method'] = match_info.group('method')
        entry['protocol'] = match_info.group('protocol')
        entry['status'] = match_info.group('status')
        entry['status'] += ' ' + httplib.responses[int(entry['status'])]
        if match_info.group('bytes') != '-':
            entry['bytes'] = match_info.group('bytes')
        if match_info.group('referrer') != '"-"':
            entry['referrer'] = match_info.group('referrer')
        entry['client'] = match_info.group('client')
        entries.append(entry)

    return json.dumps({'items': entries}, indent=4)

"""
#Geolocation support
            entry['latlong'] = ''
            if True:
            #if locateipfor and entry['request_path'].find(locateipfor) != -1:
                result = ip2geo(entry['ip'], db, log)
                if result is not None:
                    entry.update(result)
            ids.add(entry['id'])
            entries.append(entry)
        else:
            print >> log, 'Unable to parse line: ', line

"""



"""Start up an Akara server on the command-line

This is an internal module not for use by other libraries.

"""
import os
import sys
import socket
import logging
import signal

import akara
from akara import read_config
from akara import logger, logger_config
from akara.multiprocess_http import AkaraPreforkServer
from akara import global_config


# Need this in order to install "/" as service.list_services.  I think
# this code is somewhat clunky. There should be no reason to need the
# import here, but something has to install the top-level search
# handler, and doing it via a simple_service just makes things, well,
# simple. But the registry can't import services because services
# needs a fully-loaded registry to register things.
# Threw my hands up in the sky and broke the reference here.
# Caveat emptor.

from akara import services


def save_pid(pid_file):
    "Save the current pid to the given PID filename"
    # One line, newline terminated
    pid_s = str(os.getpid()) + "\n"

    try:
        f = open(pid_file, "w")
    except Exception, error:
        raise Exception("Unable to open PID file: %s" %
                        (error,))
    try:
        try:
            f.write(pid_s)
        except Exception, error:
            raise Exception("Unable to write to PID file %r: %s" %
                            (pid_file, error))
    finally:
        f.close()

def remove_pid(pid_file):
    "Remove the given filename (which should be the PID file)"
    try:
        os.remove(pid_file)
    except Exception, error:
        if not os.path.exists(pid_file):
            logger.error("Unable to remove PID file %r: %s",
                      pid_file, error)
    else:
        logger.info("Removed PID file %r", pid_file)


# There are two ways to run the Akara server, either in debug mode
# (running in the foreground, with the -X option) or in daemon mode
# (running in the background) which is the default. The latter is
# trickier to support.

# In that case the command-line program spawns off a new process,
# which is the master HTTP node ("the flup server"). It manages the
# subprocesses which actually handle the HTTP requests. The flup
# server starts up and either manages to set things up or fails
# because of some problem. The command-line program needs to exit with
# an error code if there was a problem, so there must be some sort of
# communications between the two.

# The solution is simple. Setup a pipe. The child sends either
# "success\n" or "failure\n" as appropriate. The parent (which is the
# command-line program) waits until it gets one of those messages.
class NotifyParent(object):
    def __init__(self):
        self.r_pipe, self.w_pipe = os.pipe()
    def failure(self):
        "Called in the child, when it must abort"
        os.write(self.w_pipe, "failure\n")
    def success(self):
        "Called in the child, when it's ready for HTTP requests"
        os.write(self.w_pipe, "success\n")
    def read_and_close(self):
        "Called in the parent, to wait for the child"
        status = os.read(self.r_pipe, 1000)
        os.close(self.r_pipe)
        os.close(self.w_pipe)
        return status

# Life is much easier in debug mode. There's no need to communicate
# anything to the non-existent parent.
class NoParent(object):
    def failure(self):
        pass
    def success(self):
        pass


def demonize():
    notify_parent = NotifyParent()

    if os.fork():
        # In the command-line parent. Wait for child status.
        status = notify_parent.read_and_close()

        if status.startswith("success"):
            raise SystemExit(0)
        else:
            raise SystemExit(1)

    # In the child, which is the flup server.
    try:
        # Create a new session with this process as the group leader
        try:
            setsid = os.setsid
        except AttributeError:
            os.setpgid(0, 0)
        else:
            setsid()
    except:
        notify_parent.failure()
        raise
    return notify_parent

# Sets up the global_config module contents
def set_global_config(settings):
    for name, value in settings.items():
        setattr(global_config, name, value)

def main(args):
    config_filename = args.config_filename
    debug = args.debug
    skip_pid_check = args.skip_pid_check

    first_time = True
    old_server_address = None
    sock = None
    while 1:
        # This is the main loop for the flup server.

        # Why is it a loop? A SIGHUP sent to the server
        # will shut down flup then reread the configuration
        # file, reload the extension modules, and start
        # the flup server again.

        try:
            settings, config = read_config.read_config(config_filename)
        except read_config.Error, err:
            logger.fatal(str(err))
            if first_time:
                raise SystemExit("Cannot start Akara. Exiting.")
            else:
                raise SystemExit("Cannot restart Akara. Exiting.")

        akara.raw_config = config

        # Establish the global configuration module
        set_global_config(settings)

        # In debug mode (-X), display all log messages.
        # Otherwise, use the configuration level
        if debug:
            logger.setLevel(logging.DEBUG)
        else:
            logger.setLevel(settings["log_level"])

        # Open this now, so any errors can be reported
        try:
            logger_config.set_logfile(settings["error_log"])
        except IOError, err:
            # Only give the 'akara setup' text here because it's where
            # we get to with an empty/nonexistant configuration file.
            logger.fatal("""\
Could not open the Akara error log:
   %s
Does that directory exist and is it writeable?
You may want to use 'akara setup' to set up the directory structure.""" % err)
            sys.exit(1)

        # Configure the access log
        try:
            logger_config.set_access_logfile(settings["access_log"])
        except IOError, err:
            logger.fatal("""\
Could not open the Akara access log:
   %s
Does that directory exist and is it writeable?""" % err)
            sys.exit(1)


        # Don't start if the PID file already exists.
        pid_file = settings["pid_file"]

        if first_time and (not skip_pid_check) and os.path.exists(pid_file):
            msg = ("Akara PID file %r already exists. Is another Akara instance running?\n"
                   "If not, remove the file or use the '-f' option to skip this check")
            logger.fatal(msg % (pid_file,))
            raise SystemExit(1)

        if debug or not first_time:
            notify_parent = NoParent()
        else:
            # Spawn off the actual listener.
            # The parent will always raise an exception, and never return.
            try:
                notify_parent = demonize()
            except Exception, err:
                # This can come from the parent or the child.
                logger.critical("Cannot spawn HTTP server", exc_info=True)
                raise SystemExit("Exiting - check the log file for details")


        # At this point we are in the child. Set things up as
        # far as we can go, then tell the parent that we're ready.
        try:
            server_address = settings["server_address"]
            if server_address != old_server_address:
                if sock is not None:
                    sock.close()
                sock = socket.socket()
                # XXX Should SO_REUSEADDR be a configuration setting?
                sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                # Disable Nagle's algorithm, which causes problems with
                # keep-alive. See:
                #     http://stackoverflow.com/questions/1781766/
                sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                host, port = settings["server_address"]
                if host:
                    description = "interface %r port %r" % (host, port)
                else:
                    description = "port %r" % (port,)
                try:
                    sock.bind(settings["server_address"])
                except socket.error, error:
                    raise SystemExit("Can not bind to " + description)
                logger.info("Listening to " + description)
                                      
                sock.listen(socket.SOMAXCONN)
                old_server_address = server_address

            # NOTE: StartServers not currently supported and likely won't be.
            # Why? Because the old algorithm would add/cull the server count
            # within a few check intervals (each about 1 second), so it
            # didn't have much long-term effect.
            logger.info("Akara server is running")
            server = AkaraPreforkServer(
                minSpare = settings["min_spare_servers"],
                maxSpare = settings["max_spare_servers"],
                maxChildren = settings["max_servers"],
                maxRequests = settings["max_requests_per_server"],
                settings = settings,
                config = config,
                )

            # Everything is ready to go, except for saving the PID file
            if first_time:
                save_pid(pid_file)
        except:
            notify_parent.failure()
            logger.critical("Could not set up the Akara HTTP server", exc_info=True)
            raise SystemExit("Akara HTTP server exiting - check the log file for details")

        else:
            notify_parent.success()

        # Fully demonize - no more logging to sys.std*
        # Close the standard file descriptors.
        # Redirect sys.std* to the log file
        if first_time and not debug:
            logger_config.remove_logging_to_stderr()
            logger_config.redirect_stdio()

        try:
            hupReceived = server.run(sock)
        except SystemExit:
            # Propogate the SystemExit through the system.  Remember,
            # this is also the root of the call tree for the child
            # which handles the request. The child exits at some point.
            raise

        # Strange. Why didn't flup disable this alarm?
        signal.alarm(0)
    
        if not hupReceived:
            logger.info("Akara server shutting down.")
            break
        logger.info("Akara server is restarting.")
        first_time = False
    remove_pid(pid_file)



# -*- coding: iso-8859-1 -*-
# 
"""

See: http://wiki.xml3k.org/Akara/Services/MoinCMS

Copyright 2009 Uche Ogbuji
This file is part of the open source Akara project,
provided under the Apache 2.0 license.
See the files LICENSE and NOTICE for details.
Project home, documentation, distributions: http://wiki.xml3k.org/Akara

@copyright: 2009 by Uche ogbuji <uche@ogbuji.net>
"""

__all__ = [
    "WIKITEXT_IMT", "DOCBOOK_IMT", "RDF_IMT", "HTML_IMT", "ATTACHMENTS_IMT",
    "ORIG_BASE_HEADER", "ATTACHMENTS_MODEL_XML", "ATTACHMENTS_MODEL",
    "MOIN_DOCBOOK_MODEL_XML", "MOIN_DOCBOOK_MODEL", "WIKI_RELATIVE_HEADER"
]

#import pprint
import os
import stat  # index constants for os.stat()
import re
import httplib, urllib, urllib2, cookielib
import datetime
from gettext import gettext as _

from dateutil.parser import parse as dateparse

import amara
from amara import bindery
from amara.namespaces import *
from amara.xslt import transform
from amara.writers.struct import *
from amara.bindery.html import parse as htmlparse
from amara.lib import U
from amara.lib.date import timezone, UTC
from amara.lib.iri import split_uri_ref, split_fragment, relativize, absolutize, IriError, join, is_absolute
from amara.bindery.model import examplotron_model, generate_metadata, metadata_dict
from amara.bindery.util import dispatcher, node_handler, property_sequence_getter

from akara import logger


WIKITEXT_IMT = 'text/plain'
HTML_IMT = 'text/html'
DOCBOOK_IMT = 'application/docbook+xml'
RDF_IMT = 'application/rdf+xml'
ATTACHMENTS_IMT = 'application/x-moin-attachments+xml'
ORIG_BASE_HEADER = 'x-akara-wrapped-moin'
WIKI_RELATIVE_HEADER = 'x-wiki-relative-location'

#Note: this requires the user to install the demo/etc/application_xml.py (a fixed Moin XML formatter) in plugin/formatter/ for th eMoin instance
XML_IMT = 'application/xml'

# XML models

ATTACHMENTS_MODEL_XML = '''<?xml version="1.0" encoding="UTF-8"?>
<attachments xmlns:eg="http://examplotron.org/0/" xmlns:ak="http://purl.org/xml3k/akara/xmlmodel">
  <attachment href="" ak:rel="name()" ak:value="@href"/>
</attachments>
'''

ATTACHMENTS_MODEL = examplotron_model(ATTACHMENTS_MODEL_XML)

HISTORY_MODEL_XML = '''<?xml version="1.0" encoding="UTF-8"?>
<history xmlns:eg="http://examplotron.org/0/" xmlns:ak="http://purl.org/xml3k/akara/xmlmodel">
  <rev eg:occurs="*" editor="akara" date="2010-05-12T21:35:34" id="20"/>
</history>
'''

HISTORY_MODEL = examplotron_model(HISTORY_MODEL_XML)

MOIN_DOCBOOK_MODEL_XML = '''<?xml version="1.0" encoding="UTF-8"?>
<article xmlns:eg="http://examplotron.org/0/" xmlns:ak="http://purl.org/xml3k/akara/xmlmodel" ak:resource="">
  <ak:rel name="'ak-old-type'" ak:value="glosslist[1]/glossentry[glossterm='akara:type']/glossdef//ulink/@url"/>
  <ak:rel name="'ak-type'" ak:value="section[title='akara:metadata']/glosslist/glossentry[glossterm='akara:type']/glossdef//ulink/@url"/>
  <ak:rel name="'ak-updated'" ak:value="articleinfo/revhistory/revision[1]/date"/>
  <articleinfo>
    <title ak:rel="name()" ak:value=".">FrontPage</title>
    <revhistory>
      <revision eg:occurs="*">
        <revnumber>15</revnumber>
        <date>2009-02-22 07:45:22</date>
        <authorinitials>localhost</authorinitials>
      </revision>
    </revhistory>
  </articleinfo>
  <section eg:occurs="*" ak:resource="">
    <title ak:rel="name()" ak:value=".">A page</title>
    <para>
    Using: <ulink url="http://moinmo.in/DesktopEdition"/> set <code>interface = ''</code>)
    </para>
    <itemizedlist>
      <listitem>
        <para>
          <ulink url="http://localhost:8080/Developer#">Developer</ulink> </para>
      </listitem>
    </itemizedlist>
  </section>
</article>
'''

MOIN_DOCBOOK_MODEL = examplotron_model(MOIN_DOCBOOK_MODEL_XML)


#python akara/services/moincms.py -p "Site.*" http://localhost:8880/ ~/tmp/ http://localhost:8080/
#
#Detailed license and copyright information: http://4suite.org/COPYRIGHT

#aname = partial(property_sequence_getter, u"name")
#aemail = partial(property_sequence_getter, u"email")
#auri = partial(property_sequence_getter, u"uri")

AKARA_NS = u'http://purl.org/dc/org/xml3k/akara'
CMS_BASE = AKARA_NS + u'/cms'

CAMELCASE_PAT = re.compile(u'(\s+)(([A-Z]+)([a-z]+)([A-Z]+)(\w+))(\s+)')

def text_to_moin(text):
    '''
    Convert text into a form where it appears as one would expect in Moin:
    * Normalize line endings
    * Escape CamelCase
    
    >>> from akara.util.moin import text_to_moin
    >>> text_to_moin(u' a AxBxCx   b\\r\\nMoreCamelCase foo') #Beware double-escaped chars for doctest
    u' a !AxBxCx   b\\n!MoreCamelCase foo'
    >>> text_to_moin(u' a ABC   b\\r\\nmoreCamelCase foo') #Beware double-escaped chars for doctest
    u' a ABC   b\\nmoreCamelCase foo'
    '''
    text = CAMELCASE_PAT.subn(lambda m: m.group(1) + u'!' + m.group(2) + m.groups()[-1], text)[0]
    return u'\n'.join([line.rstrip() for line in text.splitlines() ])

def cleanup_text_blocks(text):
    return u'\n'.join([line.strip() for line in text.splitlines() ])

class node(object):
    '''
    Akara Moin/CMS node, a Moin wiki page that follows a template to direct workflow
    activity, including metadata extraction
    '''
    AKARA_TYPE = u'http://purl.org/xml3k/akara/cms/resource-type'
    NODES = {}
    #Processing priority
    PRIORITY = 0
    ENDPOINTS = None
    @staticmethod
    def factory(rest_uri, moin_link=None, opener=None):
        opener = opener or urllib2.build_opener()
        logger.debug('rest_uri: ' + rest_uri)
        req = urllib2.Request(rest_uri, headers={'Accept': DOCBOOK_IMT})
        resp = opener.open(req)
        doc = bindery.parse(resp, standalone=True, model=MOIN_DOCBOOK_MODEL)
        original_wiki_base = dict(resp.info())[ORIG_BASE_HEADER]
        #self.original_wiki_base = dict(resp.info())[ORIG_BASE_HEADER]
        #amara.xml_print(self.content_cache)
        metadata, first_id = metadata_dict(generate_metadata(doc))
        metadata = metadata[first_id]
        akara_type = U(metadata[u'ak-type'])
        logger.debug('Type: ' + akara_type)
        try:
            #Older Moin CMS resource types are implemented by registration to the global node.NODES
            cls = node.NODES[akara_type]
        except KeyError:
            #Newer Moin CMS resource types are implemented by discovery of a URL,
            #to which a POST request executes the desired action
            return node.ENDPOINTS and (rest_uri, akara_type, node.ENDPOINTS[akara_type], doc, metadata, original_wiki_base)
        else:
            instance = cls(rest_uri, moin_link, opener, cache=(doc, metadata, original_wiki_base))
            return instance

    #FIXME: This cache is to help eliminate unnecessary trips back to moin to get
    #The page body.  It should soon be replaced by the proposed comprehensive caching
    def __init__(self, rest_uri, moin_link, opener=None, cache=None):
        '''
        rest_uri - the full URI to the Moin/REST wrapper for this page
        relative - the URI of this page relative to the Wiki base
        '''
        self.rest_uri = rest_uri
        self.opener = opener
        self.moin_link = moin_link
        #logger.debug('Moin link: ' + moin_link)
        #logger.debug('REST URI: ' + rest_uri)
        self.cache = cache #(doc, metadata, original_wiki_base)
        return

    def load(self):
        raise NotImplementedError

    def render(self):
        raise NotImplementedError

    def up_to_date(self, force_update=False):
        '''
        Checks whether there needs to be an update of the output
        '''
        #By default just always update
        return False

    def section_titled(self, title):
        '''
        Helper to extract content from a specific section within the page
        '''
        #FIXME: rethink this "caching" business
        doc, metadata, original_wiki_base = self.cache
        #logger.debug("section_titled: " + repr(title))
        return doc.article.xml_select(u'section[title = "%s"]'%title)

    #
    def definition_list(self, list_path, contextnode=None, patterns=None):
        '''
        Helper to construct a dictionary from an indicated definition list on the page
        '''
        #FIXME: rethink this "caching" business
        #Use defaultdict instead, for performance
        patterns = patterns or {None: lambda x: U(x) if x else None}
        doc, metadata, original_wiki_base = self.cache
        contextnode = contextnode or doc.article
        top = contextnode.xml_select(list_path)
        if not top:
            return None
        #Go over the glossentries, and map from term to def, applying the matching
        #Unit transform function from the patterns dict
        result = dict((U(i.glossterm), patterns.get(U(i.glossterm), patterns[None])(i.glossdef))
                      for i in top[0].glossentry)
        #logger.debug("definition_list: " + repr(result))
        return result

node.NODES[node.AKARA_TYPE] = node


#XXX: do we really need this function indirection for simple global dict assignment?
def register_node_type(type_id, nclass):
    node.NODES[type_id] = nclass


def wiki_uri(original_base, wrapped_base, link, relative_to=None, raw=False):
    '''
    Constructs absolute URLs to the original and REST-wrapper for a page, given a link from another page
    
    original_base - The base URI of the actual Moin instance
    wrapped_base - The base URI of the REST-wrapped proxy of the Moin instance
    link - the relative link, generally from one wiki page to another
    relative_to - the REST-wrapped version of the page from which the relative link came, defaults to same as wrapped_base
    raw - the link is a full hierarchical path, rather than relative to the wiki base

    Returns a tuple (wrapped_uri, abs_link)
    
    wrapped_uri - the URI wrapped for REST ops
    abs_link - the full, original wiki URL
    
    >>> from akara.util.moin import wiki_uri
    >>> wiki_uri('http://example.com/mywiki/', 'http://localhost:8880/moin/w/', '/spam')
    ('http://localhost:8880/moin/w/spam', 'http://example.com/mywiki/spam')
    >>> wiki_uri('http://example.com/mywiki/', 'http://localhost:8880/moin/w/', 'http://google.com/spam')
    (None, None)
    >>> wiki_uri('http://example.com/mywiki/', 'http://localhost:8880/moin/w/', 'http://google.com/spam', raw=True)
    (None, None)
    >>> wiki_uri('http://example.com/mywiki/', 'http://localhost:8880/moin/w/', '/mywiki/spam', raw=True)
    ('http://localhost:8880/moin/w/spam', 'http://example.com/mywiki/spam')
    >>> wiki_uri('http://example.com/mywiki/', 'http://localhost:8880/moin/w/', '/mywiki/spam')
    ('http://localhost:8880/moin/w/mywiki/spam', 'http://example.com/mywiki/mywiki/spam')
    '''
    #rel_link = relativize(abs_link, original_wiki_base)
    #e.g. original wiki base is http://myhost:8080/mywiki/ and link is /a/b
    #abs_link is http://myhost:8080/mywiki/a/b note the need to strip the leading / to get that
    #from akara import logger; logger.debug('wiki_uri' + repr((original_base, wrapped_base, link, relative_to, absolutize(link, original_base.rstrip('/')+'/'))))
    if raw and not is_absolute(link):
        (scheme, authority, path, query, fragment) = split_uri_ref(original_base)
        link = link[len(path):]
    link = link.lstrip('/')
    abs_link = absolutize(link, original_base.rstrip('/')+'/')
    rel_to_wikibase = relativize(abs_link, original_base.rstrip('/')+'/')
    if not rel_to_wikibase:
        #It's not a relative wiki link
        return None, None
    rest_uri = absolutize(rel_to_wikibase, wrapped_base.rstrip('/')+'/')
    return rest_uri, abs_link


#
def unwrap_uri(original_base, wrapped_base, rest_uri):
    '''
    Constructs an absolute URL to the original Moin page
    
    original_base - The base URI of the actual Moin instance
    wrapped_base - The base URI of the REST-wrapped proxy of the Moin instance
    rest_uri - moinrest-wrapped URI

    Returns a tuple unwrapped_link
    
    >>> from akara.util.moin import unwrap_uri
    >>> unwrap_uri('http://example.com/mywiki/', 'http://localhost:8880/moin/w/', 'http://localhost:8880/moin/w/spam')
    'http://example.com/mywiki/spam'
    >>> unwrap_uri('http://example.com/', 'http://localhost:8880/moin/w/', 'http://localhost:8880/moin/w/spam')
    'http://example.com/spam'
    '''
    rel = relativize(rest_uri, wrapped_base.rstrip('/')+'/')
    return absolutize(rel, original_base.rstrip('/')+'/')


RE_XML_WIKISPLIT = re.compile(u'\s+')

def wiki_normalize(s):
    '''
    A smarter variety of string normalization.  Multiple runs of whitespace are replaced
    with a space, except that " \n" goes unchanged, and runs of whitespace at the beginning
    of a line go unchanged
    
    >>> from akara.util.moin import wiki_normalize
    >>> wiki_normalize(u'= A =\\n * spam \\n  * eggs\\n\\n') #Beware double-escaped chars for doctest
    
    '''
    #First of all normalize line endings
    s = '\n'.join(s.splitlines())
    def repl(m):
        if '\n' in m.group(0):
            return m.group(0)
        else:
            return ' '
    return RE_XML_WIKISPLIT.subn(repl, s)[0]




"""
Atom output

Based on:
    
    RSS Handling

    @copyright: 2006-2007 MoinMoin:ThomasWaldmann
    @license: GNU GPL, see COPYING for details.
"""
import re, time
from MoinMoin import config, wikiutil
from MoinMoin.logfile import editlog
from MoinMoin.util import timefuncs
from MoinMoin.Page import Page
from MoinMoin.wikixml.util import RssGenerator

from amara.writers.struct import structencoder, E, E_CURSOR, NS, ROOT, RAW
from amara.namespaces import ATOM_NAMESPACE

RSSWIKI_NAMESPACE = u"http://purl.org/rss/1.0/modules/wiki/"

def full_url(request, page, querystr=None, anchor=None):
    url = page.url(request, anchor=anchor, querystr=querystr)
    url = wikiutil.escape(url)
    return request.getQualifiedURL(url)

def execute(pagename, request):
    """ Send recent changes as an RSS document
    """
    cfg = request.cfg

    # get params
    items_limit = 100
    try:
        max_items = int(request.values['items'])
        max_items = min(max_items, items_limit) # not more than `items_limit`
    except (KeyError, ValueError):
        # not more than 15 items in a RSS file by default
        max_items = 15
    try:
        unique = int(request.values.get('unique', 0))
    except ValueError:
        unique = 0
    try:
        diffs = int(request.values.get('diffs', 0))
    except ValueError:
        diffs = 0
    ## ddiffs inserted by Ralf Zosel <ralf@zosel.com>, 04.12.2003
    try:
        ddiffs = int(request.values.get('ddiffs', 0))
    except ValueError:
        ddiffs = 0

    urlfilter = request.values.get('filter')
    if urlfilter:
        urlfilter = re.compile(urlfilter)
    else:
        urlfilter = None

    # get data
    log = editlog.EditLog(request)
    logdata = []
    counter = 0
    pages = {}
    lastmod = 0
    for line in log.reverse():
        if urlfilter and not(urlfilter.match(line.pagename)):
            continue
        if not request.user.may.read(line.pagename):
            continue
        if (not line.action.startswith('SAVE') or
            ((line.pagename in pages) and unique)): continue
        #if log.dayChanged() and log.daycount > _MAX_DAYS: break
        line.editor = line.getInterwikiEditorData(request)
        line.time = timefuncs.tmtuple(wikiutil.version2timestamp(line.ed_time_usecs)) # UTC
        logdata.append(line)
        pages[line.pagename] = None

        if not lastmod:
            lastmod = wikiutil.version2timestamp(line.ed_time_usecs)

        counter += 1
        if counter >= max_items:
            break
    del log

    timestamp = timefuncs.formathttpdate(lastmod)
    etag = "%d-%d-%d-%d-%d" % (lastmod, max_items, diffs, ddiffs, unique)

    # for 304, we look at if-modified-since and if-none-match headers,
    # one of them must match and the other is either not there or must match.
    if request.if_modified_since == timestamp:
        if request.if_none_match:
            if request.if_none_match == etag:
                request.status_code = 304
        else:
            request.status_code = 304
    elif request.if_none_match == etag:
        if request.if_modified_since:
            if request.if_modified_since == timestamp:
                request.status_code = 304
        else:
            request.status_code = 304
    else:
        # generate an Expires header, using whatever setting the admin
        # defined for suggested cache lifetime of the RecentChanges RSS doc
        expires = time.time() + cfg.rss_cache

        request.mimetype = 'application/rss+xml'
        request.expires = expires
        request.last_modified = lastmod
        request.headers['Etag'] = etag

        # send the generated XML document
        baseurl = request.url_root

        logo = re.search(r'src="([^"]*)"', cfg.logo_string)
        if logo:
            logo = request.getQualifiedURL(logo.group(1))

        # prepare output
        output = structencoder(indent=u"yes")

        FEED_HEADER_COMMENT = '''
<!--
    Add an "items=nnn" URL parameter to get more than the default 15 items.
    You cannot get more than %d items though.
    
    Add "unique=1" to get a list of changes where page names are unique,
    i.e. where only the latest change of each page is reflected.
    Add "diffs=1" to add change diffs to the description of each items.
    
    Add "ddiffs=1" to link directly to the diff (good for FeedReader).
    Current settings: items=%i, unique=%i, diffs=%i, ddiffs=%i
-->
        ''' % (items_limit, max_items, unique, diffs, ddiffs)

        # Feed envelope
        page = Page(request, pagename)
        f = output.cofeed(
            ROOT(
                NS(u'', ATOM_NAMESPACE),
                NS(u'wiki', RSSWIKI_NAMESPACE),
                E_CURSOR((ATOM_NAMESPACE, u'feed'),
              )
            )
        )
        f.send(E((ATOM_NAMESPACE, u'id'), full_url(request, page).encode(config.charset))),
        f.send(E((ATOM_NAMESPACE, u'title'), cfg.sitename.encode(config.charset))),
        f.send(E((ATOM_NAMESPACE, u'link'), {u'href': request.url_root.encode(config.charset)})),
        f.send(E((ATOM_NAMESPACE, u'summary'), ('RecentChanges at %s' % cfg.sitename).encode(config.charset))),
        #Icon
        #E((ATOM_NAMESPACE, u'link'), {u'href': logo.encode(config.charset)}),

        #if cfg.interwikiname:
        #    handler.simpleNode(('wiki', 'interwiki'), cfg.interwikiname)

        for item in logdata:
            anchor = "%04d%02d%02d%02d%02d%02d" % item.time[:6]
            page = Page(request, item.pagename)
            #link = full_url(request, page, anchor=anchor)
            if ddiffs:
                link = full_url(request, page, querystr={'action': 'diff'})
            else:
                link = full_url(request, page)

            # description
            desc_text = item.comment
            if diffs:
                # TODO: rewrite / extend wikiutil.pagediff
                # searching for the matching pages doesn't really belong here
                revisions = page.getRevList()

                rl = len(revisions)
                for idx in range(rl):
                    rev = revisions[idx]
                    if rev <= item.rev:
                        if idx + 1 < rl:
                            lines = wikiutil.pagediff(request, item.pagename, revisions[idx+1], item.pagename, 0, ignorews=1)
                            if len(lines) > 20:
                                lines = lines[:20] + ['...\n']
                            lines = '\n'.join(lines)
                            lines = wikiutil.escape(lines)
                            desc_text = '%s\n<pre>\n%s\n</pre>\n' % (desc_text, lines)
                        break
            #if desc_text:
            #    handler.simpleNode('description', desc_text)

            # contributor
            edattr = {}
            #if cfg.show_hosts:
            #    edattr[(handler.xmlns['wiki'], 'host')] = item.hostname
            if item.editor[0] == 'interwiki':
                edname = "%s:%s" % item.editor[1]
                ##edattr[(None, 'link')] = baseurl + wikiutil.quoteWikiname(edname)
            else: # 'ip'
                edname = item.editor[1]
                ##edattr[(None, 'link')] = link + "?action=info"

            history_link = full_url(request, page, querystr={'action': 'info'})

            f.send(
                E((ATOM_NAMESPACE, u'entry'),
                    E((ATOM_NAMESPACE, u'id'), link.encode(config.charset)),
                    E((ATOM_NAMESPACE, u'title'), item.pagename.encode(config.charset)),
                    E((ATOM_NAMESPACE, u'updated'), timefuncs.W3CDate(item.time).encode(config.charset)),
                    E((ATOM_NAMESPACE, u'link'), {u'href': link.encode(config.charset)}),
                    E((ATOM_NAMESPACE, u'summary'), desc_text.encode(config.charset)),
                    E((ATOM_NAMESPACE, u'author'),
                        E((ATOM_NAMESPACE, u'name'), edname.encode(config.charset))
                    ),
                    #E((ATOM_NAMESPACE, u'title'), item.pagename.encode(config.charset)),
                    # wiki extensions
                    E((RSSWIKI_NAMESPACE, u'wiki:version'), ("%i" % (item.ed_time_usecs)).encode(config.charset)),
                    E((RSSWIKI_NAMESPACE, u'wiki:status'), (u'deleted', u'updated')[page.exists()]),
                    E((RSSWIKI_NAMESPACE, u'wiki:diff'), link.encode(config.charset)),
                    E((RSSWIKI_NAMESPACE, u'wiki:history'), history_link.encode(config.charset)),
                    # handler.simpleNode(('wiki', 'importance'), ) # ( major | minor )
                    # handler.simpleNode(('wiki', 'version'), ) # ( #PCDATA )
                )
            )

        # emit logo data
        #if logo:
        #    handler.startNode('image', attr={
        #        (handler.xmlns['rdf'], 'about'): logo,
        #        })
        #    handler.simpleNode('title', cfg.sitename)
        #    handler.simpleNode('link', baseurl)
        #    handler.simpleNode('url', logo)
        #    handler.endNode('image')

        f.close()
        request.write(output.read())




"""Demo of several ways to work with Amara's Atom tools

Some configurations can be changed in akara.conf. The default settings are:

class atomtools:
    entries = "/path/to/entry/files/*.atom"
    feed_envelope = '''<feed xmlns="http://www.w3.org/2005/Atom">
<title>This is my feed</title><id>http://example.com/my_feed</id>
</feed>'''

'entries' is the glob path to a set of Atom entries, where the root
element to each XML document must be "entry" in the Atom
namespace). The "feed_envelope" goes around the entries to make the
full Atom feed.  The entries are listed after the <title>.

"""
# "Make Emacs happy with a close quote. Otherwise it gets confused.

from datetime import datetime, timedelta
import glob
from itertools import dropwhile

import amara
from amara import bindery
from amara.tools import atomtools
from amara.thirdparty import httplib2
from amara.lib.util import first_item
from amara.thirdparty import json

from akara.services import simple_service
from akara import request, response
from akara import logger, module_config


# These come from the akara.demos.atomtools section of the Akara configuration file
ENTRIES = module_config().warn("entries", "/path/to/entry/files/*.atom",
                               "glob path to Atom entries")

FEED_ENVELOPE = module_config().warn("feed_envelope",
'''<feed xmlns="http://www.w3.org/2005/Atom">
<title>This is my feed</title><id>http://example.com/my_feed</id>
</feed>''', "XML envelope around the Atom entries")


#text/uri-list from RFC 2483
SERVICE_ID = 'http://purl.org/akara/services/demo/atom.json'
@simple_service('GET', SERVICE_ID, 'akara.atom.json', 'application/json')
def atom_json(url):
    '''
    Convert Atom syntax to Exhibit JSON
    (see: http://www.ibm.com/developerworks/web/library/wa-realweb6/ ; this is based on listing 3)
    
    Sample requests:
    * curl "http://localhost:8880/akara.atom.json?url=url=http://zepheira.com/feeds/news.atom"
    * curl "http://localhost:8880/akara.atom.json?url=http://picasaweb.google.com/data/feed/base/user/dysryi/albumid/5342439351589940049"
    * curl "http://localhost:8880/akara.atom.json?url=http://earthquake.usgs.gov/eqcenter/catalogs/7day-M2.5.xml"
    '''
    entries = atomtools.ejsonize(url)
    return json.dumps({'items': entries}, indent=4)

# This uses a simple caching mechanism.
# If the cache is over 15 minutes old then rebuild the cache.
DOC_CACHE = None
def _need_refresh():
    if DOC_CACHE is None:
        return True
    if datetime.now() > DOC_CACHE[1]: # check for expiration
        return True
    return False

SERVICE_ID = 'http://purl.org/akara/services/demo/aggregate.atom'
@simple_service('GET', SERVICE_ID, 'akara.aggregate.atom', str(atomtools.ATOM_IMT))
def aggregate_atom():
    """Aggregate a set of Atom entries and return as an Atom feed
    
    Sample request:
    * curl "http://localhost:8880/akara.aggregate.atom"
    """
    global DOC_CACHE
    if _need_refresh():
        filenames = glob.glob(ENTRIES)
        doc, metadata = atomtools.aggregate_entries(FEED_ENVELOPE, filenames)
        DOC_CACHE = doc.xml_encode('xml-indent'), datetime.now() + timedelta(minutes=15)
    return DOC_CACHE[0]


# We love Atom, but for sake of practicality (and JSON fans), here is
# a transform for general feeds
SERVICE_ID = 'http://purl.org/akara/services/demo/webfeed.json'
@simple_service('GET', SERVICE_ID, 'akara.webfeed.json', 'application/json')
def webfeed_json(url):
    """Convert an Atom feed to Exhibit JSON
    
    Sample request:
    * curl "http://localhost:8880/akara.webfeed.json?url=http://feeds.delicious.com/v2/rss/recent%3Fmin=1%26count=15"
    * curl http://localhost:8880/akara.webfeed.json?url=http://localhost:8880/akara.aggregate.atom
    """
    import feedparser   # From http://www.feedparser.org/

    feed = feedparser.parse(url)
    # Note: bad URLs might mean the feed doesn't have headers
    
    def process_entry(e):
        data = {
            u'id': e.link,
            u'label': e.link,
            u'title': e.title,
            u'link': e.link,
            u'updated': e.updated,
        }
        #Optional bits
        if 'content' in data:
            data[u'content'] = e.content
        if 'description' in data:
            data[u'description'] = e.description
        if 'author_detail' in data:
            data[u'author_name'] = e.author_detail.name
        return data

    entries = [ process_entry(e) for e in feed.entries ]
    return json.dumps({'items': entries}, indent=4)

RDF_IMT = 'application/rdf+xml'
ATOM_IMT = 'application/atom+xml'

# Read RSS2, and generate Atom or other format
SERVICE_ID = 'http://purl.org/akara/services/demo/rss2translate'
@simple_service('GET', SERVICE_ID, 'akara.rss2translate')
def rss2translate(url=None, format=None):
    """Convert RSS 2.0 feed to Atom or RSS 1.0
    
    Sample request:
    * curl "http://localhost:8880/akara.rss2translate?url=http://feeds.delicious.com/v2/rss/recent"

    This is a demo and is not meant as an industrial-strength converter.
    """
    # Support connection-negotiation in addition to query parameter
    if not format:
        accepted_imts = request.environ.get('HTTP_ACCEPT', '').split(',')
        imt = first_item(dropwhile(lambda x: '*' in x, accepted_imts))
        if imt == 'RDF_IMT':
            format = 'rss1'
        else:
            format = 'atom'
    
    if not url:
        raise AssertionError("The 'url' query parameter is mandatory.")

    import feedparser # From http://www.feedparser.org/
    feed = feedparser.parse(url)
    
    # Note: bad URLs might mean the feed doesn't have headers
    logger.debug('Feed info: ' + repr((url, feed.version, feed.encoding, feed.headers.get('Content-type'))))

    updated = getattr(feed.feed, 'updated_parsed', None)
    if updated:
        #FIXME: Double-check this conversion
        updated = datetime(*updated[:7]).isoformat()
    
    f = atomtools.feed(title=feed.feed.title, updated=updated, id=feed.feed.link)
    for e in feed.entries:
        updated = getattr(e, 'updated_parsed', None)
        if updated:
            #FIXME: Double-check this conversion
            updated = datetime(*updated[:7]).isoformat()
        links = [
            #FIXME: self?
            (e.link, u'alternate'),
        ]
        f.append(
            e.link,
            e.title,
            updated = updated,
            summary=e.description,
            #e.author_detail.name
            #authors=authors,
            links=links,
        )

    if format == 'atom':
        result = f.xml_encode()
        response.add_header("Content-Type", ATOM_IMT)
    else:
        result = f.rss1format()
        response.add_header("Content-Type", RDF_IMT)
    return result




"""Support code for building service pipelines

A pipeline is a collection of stages. Each stage corresponds to a
service, which takes the input, processes it, and produces output.

The first stage uses the incoming HTTP request as input, and the
output from the last stage produces the HTTP response. All of the
stages follow the WSGI specification. The pipeline knows how to
convert the WSGI response into the successive WSGI query.

"""

__all__ = ["Pipeline", "Stage", "register_pipeline"]

import urllib
from cStringIO import StringIO

from akara import logger
from akara import registry

# Helper function to figure out which stage is the first and/or last stage
#  [X] -> [ (1,1,X])
#  [X, Y] -> [ (1,0,X), (0,1,Y) ]
#  [X, Y, Z] -> [ (1,0,X), (0,0,Y), (0,1,Z) ]
def _flag_position(data):
    first_flags = [1] + [0] * (len(data)-1)
    last_flags = first_flags[::-1]
    return zip(range(len(data)), first_flags, last_flags, data)

def _find_header(search_term, headers):
    search_term = search_term.lower()
    for (name, value) in headers:
        if name.lower() == search_term:
            return value
    raise AssertionError("Could not find %r in the headers" % search_term)


class Pipeline(object):
    def __init__(self, ident, path, stages, doc):
        if not len(stages):
            raise TypeError("No stages defined for the pipeline")
        self.ident = ident
        self.path = path
        self.stages = stages
        self.doc = doc

    def __call__(self, environ, start_response):
        # The WSGI entry point for the pipeline
        
        logger.debug("Started the %s (%s) pipeline", self.ident, self.path)

        # Help capture the response of a WSGI request,
        # so I can forward it as input to the next request.
        captured_response = [None, None, None]
        captured_body_length = None
        def capture_start_response(status, headers, exc_info=None):
            if exc_info is None:
                captured_response[:] = [status, headers, False]
            else:
                captured_response[:] = [status, headers, True]
                # Forward this to the real start_response
                return start_response(status, headers, exc_info)

        num_stages = len(self.stages)
        for stage_index, is_first, is_last, stage in _flag_position(self.stages):
            service = registry.get_a_service_by_id(stage.ident)
            if service is None:
                logger.error("Pipeline %r(%r) could not find a %r service",
                              self.ident, self.path, stage.ident)
                start_response("500 Internal server error", [("Content-Type", "text/plain")])
                return ["Broken internal pipeline.\n"]

            # Construct a new environ for each stage in the pipeline.
            # We have to make a new one since a stage is free to
            # do whatever it wants to the environ. (It does not have
            # free reign over all the contents of the environ.)
            stage_environ = environ.copy()
            if not is_first:
                # The first stage gets the HTTP request method
                # Everything else gets a POST.
                stage_environ["REQUEST_METHOD"] = "POST"
            assert service.path is not None # Can some services/pipelines not be mounted?
            stage_environ["SCRIPT_NAME"] = service.path
            #stage_environ["PATH_INFO"] = ... # I think  this is best left unchanged. XXX

            if is_first:
                # Augment the QUERY_STRING string with any pipeline-defined query string
                # (You probably shouldn't be doing this. Remove this feature? XXX)
                if stage.query_string:
                    if stage_environ["QUERY_STRING"]:
                        stage_environ["QUERY_STRING"] += ("&" + stage.query_string)
                    else:
                        stage_environ["QUERY_STRING"] = stage.query_string
            else:
                # The other stages get nothing about the HTTP query string
                # but may get a pipeline-defined query string
                if stage.query_string:
                    stage_environ["QUERY_STRING"] = stage.query_string
                else:
                    stage_environ["QUERY_STRING"] = ""

            if not is_first:
                # Forward information from the previous stage
                stage_environ["CONTENT_TYPE"] = _find_header("content-type",
                                                             captured_response[1])
                stage_environ["CONTENT_LENGTH"] = captured_body_length
                stage_environ["wsgi.input"] = captured_body

                # Make the previous response headers available to the next stage
                stage_environ["akara.pipeline_headers"] = captured_response[1]

            logger.debug("Pipeline %r(%r) at stage %r (%d/%d)",
                         self.ident, self.path, stage.ident, stage_index+1, num_stages)
            if is_last:
                # End of the pipeline. Let someone else deal with the response
                return service.handler(stage_environ, start_response)
            else:
                # Intermediate stage output. Collect to forward to the next stage
                captured_body = StringIO()
                result = service.handler(stage_environ, capture_start_response)

                # Did start_response get an exc_info term? (It might not
                # have been thrown when forwarded to the real start_response.)
                if captured_response[2]:
                    # It didn't raise an exception. Assume the response contains
                    # the error message. Forward it and stop the pipeline.
                    logger.debug(
                        "Pipeline %r(%r) start_response received exc_info from stage %r. Stopping.",
                        self.ident, self.path, stage.ident)
                    return result

                # Was there some sort of HTTP error?
                status = captured_response[0].split(None, 1)[0]
                # XXX What counts as an error?
                if status not in ("200", "201"):
                    logger.debug(
                        "Pipeline %r(%r) start_response received status %r from stage %r. Stopping.",
                        self.ident, self.path, status, stage.ident)
                    start_response(captured_response[0], captured_response[1])
                    # This should contain error information
                    return result

                # Save the response to the cStringIO
                try:
                    # We might be able to get some better performance using
                    # a wsgi.file_wrapper. If the chunks come from a file-like
                    # object then we can reach in and get that file-like object
                    # instead of copying it to a new one
                    for chunk in result:
                        captured_body.write(chunk)
                finally:
                    # Part of the WSGI spec
                    if hasattr(result, "close"):
                        result.close()
                captured_body_length = captured_body.tell()
                captured_body.seek(0)

        raise AssertionErorr("should never get here")


# The dictionary values may be strings for single-valued arguments, or
# list/tuples for multiple-valued arguments. That is
#   dict(a=1, z=9)      -> "a=1&z=9"
#   dict(a=[1,2, z=9])  -> "a=1&a=2&z=9"
# This function helps flatten the values, producing a tuple-stream
# that urlencode knows how to process.

def _flatten_kwargs_values(kwargs):
    result = []
    if isinstance(kwargs, dict):
        args = kwargs.items()
    else:
        args = kwargs
    for k,v in args:
        if isinstance(v, basestring):
            result.append( (k,v) )
        else:
            for item in v:
                result.append( (k, item) )
    return result

def _build_query_string(query_args, kwargs):
    if query_args is None:
        if kwargs is None or kwargs == {}:
            return ""
        # all kwargs MUST be url-encodable
        return urllib.urlencode(_flatten_kwargs_values(kwargs))

    if kwargs is None or kwargs == {}:
        # query_args MUST be url-encodable
        return urllib.urlencode(_flatten_kwargs_values(query_args))
    raise TypeError("Cannot specify both 'query_args' and keyword arguments")


class Stage(object):
    """Define a stage in the pipeline

    'ident' is the service identifier which uniquely identifies the service
    To specify additional QUERY_STRING arguments passed to the service use one of:

      query_args - a list of (name, value) tuples
      **kwargs - the kwargs.items() are used as (name, value) tuples
      query_string - the raw query string

    If the value is a string then the name, value pair is converted to
    an HTTP query parameter. Otherwise the value is treated as a list
    and each list item adds a new query parameter

            (name, value[0]), (name, value[1]), ...

    Here are examples:
       # Using query_args (which preserves order)
       Stage("http://example.com", [("a", ["1", "2"]), ("b", "9")])
           ->  QUERY_STRING = "a=1&a=2&b=9"

       # Using kwargs (which might not preserve order)
       Stage("http://example.com", a=["1", "2"], b="9")
           ->  QUERY_STRING = "b=9&a=1&a=2"

       # Using a raw query string
       Stage("http://example.com", query_string="a=2&b=9&a=1")
           ->  QUERY_STRING = "a=2&b=9&a=1"
    
    The first stage gets the HTTP request QUERY_STRING plus
    the query string defined for the stage. The other stages
    only get the query string defined for the stage.
    """
    def __init__(self, ident, query_args=None, query_string=None, **kwargs):
        self.ident = ident
        if query_string is not None:
            if query_args is not None:
                raise TypeError("Cannot specify both 'query_string' and 'query_args'")
            if kwargs:
                raise TypeError("Cannot specify both 'query_string' and keyword argument")
            self.query_string = query_string
        else:
            self.query_string = _build_query_string(query_args, kwargs)


def _normalize_stage(stage):
    if isinstance(stage, basestring):
        return Stage(stage)
    return stage

def register_pipeline(ident, path=None, stages=None, doc=None):
    if not stages:
        raise TypeError("a pipeline must have stages")
    stages = [_normalize_stage(stage) for stage in stages]

    # Should I check that the dependent stages are already registered?

    pipeline = Pipeline(ident, path, stages, doc)
    registry.register_service(ident, path, pipeline, doc)



# -*- encoding: utf-8 -*-
'''
@ 2011 by Uche ogbuji <uche@ogbuji.net>

This file is part of the open source Akara project,
provided under the Apache 2.0 license.
See the files LICENSE and NOTICE for details.
Project home, documentation, distributions: http://wiki.xml3k.org/Akara

 Module name:: akara.demo.cache_proxy

Proxies a remote URL to improve the cacheability of GET requests

= Defined REST entry points =

http://purl.org/akara/services/demo/cache-proxy (akara.cache-proxy) Handles GET

= Configuration =

class cache_proxy:
    maxlen = {
      None: 3600,
      "http://poems.com": 24*3600,
      "http://poemtree.com": 24*3600,
      "http://www.poemtree.com": 24*3600,
    }
    override_stale = 1

= Notes on security =

This module makes a remote GET request, and rewrites the response headers 

= Behaviour =

If 'override_stale' is 1, the module will make the response cacheable
independent of the desires of the downstream server as expressed in its response
headers.  If 0 (the default), any attempt by the downstream server to
explicitly mark a response as stale will be respected (passed upstream), but
otherwise the TTLs will be replaced by those declared in 'maxlen'.

Since the intent of this module is to improve cacheability, if the response's TTL
is greater than that specified in the maxlen configuration, then we'll use the
former.

Note: for simplicity of implementation, incoming Expires headers are converted to
the equivalent Cache-Control max-age directive.
'''

import amara
from amara.thirdparty import httplib2

import akara
from akara.services import simple_service
from akara import response
from akara import logger
from akara.util import normalize_http_header_name

import calendar
import email
import email.Utils
import time

MAXLEN = akara.module_config().get('maxlen')
if None in MAXLEN:
    DEFAULT_MAXLEN = MAXLEN[None]
    del MAXLEN[None]
else:
    DEFAULT_MAXLEN = 3600

OVERRIDE_STALE = akara.module_config().get('override_stale',0)

CACHE_PROXY_SERVICE_ID = 'http://purl.org/xml3k/akara/services/demo/cache-proxy'

MAXAGE_HEADER = lambda age: ('Cache-Control','max-age={0}'.format(age))

#FIXME: recycle after N uses
H = httplib2.Http()

def get_max_age(url):
    for k in MAXLEN:
        #XXX url normalize?
        if url.startswith(k):
            return MAXLEN[k]
            break
    else:
        return DEFAULT_MAXLEN

def is_fresh(resp):
    """
    Returns a tuple, the first element a boolean whether the response can be
    considered (for our purposes) fresh or not, and the second the freshness
    lifetime of the response.

    Much of this is reworked from httplib2._entry_disposition. We can't reuse it
    directly since it assumes responses are stale unless otherwise marked as
    fresh, and we want to do the opposite.
    """
    fresh = True
    freshness_lifetime = 0

    cc_response = httplib2._parse_cache_control(resp)
    if 'no-cache' in cc_response or 'private' in cc_response:
        fresh = False
    elif 'date' in resp:
        date = calendar.timegm(email.Utils.parsedate_tz(resp['date']))
        now = time.time()
        current_age = max(0, now - date - 5) # Give us 5 seconds to get this far
        if 'max-age' in cc_response:
            try:
                freshness_lifetime = int(cc_response['max-age'])
            except ValueError:
                freshness_lifetime = 0

        elif 'expires' in resp:
            expires = email.Utils.parsedate_tz(resp['expires'])
            if expires == None:
                freshness_lifetime = 0
            else:
                freshness_lifetime = calendar.timegm(expires) - date
        else:
            freshness_lifetime = 0

        if freshness_lifetime < current_age:
            logger.debug('lifetime = {0}, age = {1}, so marking explicitly stale'.format(freshness_lifetime,current_age))
            fresh = False

    return fresh, freshness_lifetime

@simple_service('GET', CACHE_PROXY_SERVICE_ID, 'akara.cache-proxy')
def akara_cache_proxy(url=None):
    '''
    Sample request:
    curl -I "http://localhost:8880/akara.cache-proxy?url=http://poemtree.com/poems/UsefulAdvice.htm"
    '''
    logger.debug('remote URL {0}: '.format(repr(url)))
    if not url:
        raise ValueError('url query parameter required')
    resp, content = H.request(url)

    if OVERRIDE_STALE:
        response.add_header(*MAXAGE_HEADER(get_max_age(url)))
    else:
        (fresh, lifetime) = is_fresh(resp)
        if fresh:
            response.add_header(*MAXAGE_HEADER( max(get_max_age(url),lifetime) ))
        else:
            response.add_header(*MAXAGE_HEADER(0))
    
    logger.debug('remote response headers {0}: '.format(repr(resp)))
    #Oof. What about 'transfer-encoding' and other such headers
    for k in resp:
        if k not in ('server','status', 'transfer-encoding', 'content-length','cache-control','expires','date'):
            response.add_header(normalize_http_header_name(k), resp[k])
    #response.add_header(k, resp[k])
    #FIXME: This might distort return encoding, which would of course throw off content length & encoding.  Workaround for now is removal of e.g. transfer-encoding (above)

    return content



# -*- encoding: utf-8 -*-
'''
See also: http://broadcast.oreilly.com/2009/04/data-chef-spss-tripe-consomme.html

----

Python interfaces to R
Create new topic
 
there seems to be many... e.g. RSPython, rpy, and even a redesigned version rpy2 (see http://rpy.sourceforge.net) -- what works well for you, and why?
rsvp 2 days ago
 
My *nix alternative would be to use <<HereDocument scripting with Python and R within a bash script to also get access to the shell as a bonus feature.
rsvp 2 days ago
 
Relatively new interface called "PypeR" -- see http://www.jstatsoft.org/v35/c02/paper for PDF or for the source code http://rinpy.sourceforge.net/
rsvp about 16 hours ago
 
Ryan Rosario gives a walkthrough of his data mining code using RPy2 as interface to R, http://www.bytemining.com/2010/10/accessing-r-from-python-using-rpy2/ -- the video gets going after the first ten minutes (35 minutes total running time). The caveats he points out are worth noting.

(from https://convore.com/rstats/python-interfaces-to-r/ )

'''

from __future__ import with_statement
import sys, time
import urllib, urlparse
import tempfile
import os
import re
import csv
import cgi
from cStringIO import StringIO
from gettext import gettext as _
from itertools import *
from functools import *
from subprocess import *

import amara
from amara.xslt import transform
from amara.xpath.util import simplify
from amara.bindery import html
from amara.lib.util import *
# Requires Python 2.6 or http://code.google.com/p/json/
from amara.thirdparty import json

import akara
from akara.services import simple_service

VAR_PAT = re.compile('VARIABLE\s+LABELS\s+(((\w+)\s+"([^"]+)"\s*)+)\.')
VAR_DEF_PAT = re.compile('(\w+)\s+"([^"]+)"')

VALUE_PAT = re.compile('VALUE\s+LABELS\s+((/(\w+)\s+(\'(\w+)\'\s+"([^"]+)"\s*)+)+)\.')
VALUE_DEF_SET_PAT = re.compile('/(\w+)\s+((\'(\w+)\'\s+"([^"]+)"\s*)+)')
VALUE_DEF_PAT = re.compile('\'(\w+)\'\s+"([^"]+)"')

VALUE_SET_TYPE = 'value_set'
VARIABLE_LABELS_TYPE = 'variable_labels'
VALUE_LABELS_TYPE = 'value_labels'

#R_SCRIPT = '''library(foreign)
#mydata <- read.spss(file='%s')
#write.csv2(mydata)
#'''

R_SCRIPT = '''library(Hmisc)
mydata <- spss.get(file='%s')
write.csv2(mydata)
'''

R_FILE_CMD = akara.module_config(__name__).get('r_command', 'r')

POR_REQUIRED = _("The 'POR' POST parameter is mandatory.")

SERVICE_ID = 'http://purl.org/akara/services/demo/spss.json'
@simple_service('POST', SERVICE_ID, 'spss.json', 'application/json')
def spss2json(body, ctype, **params):
    '''
    Uses GNU R to convert SPSS to JSON
    Optionally tries to guess long labels from an original .SPS file
    
    Requires POST body of multipart/form-data
    
    Sample request:
    curl -F "POR=@foo.por" http://localhost:8880/spss.json
    curl -F "POR=@foo.por" -F "SPSS=@foo.sps" http://localhost:8880/spss.json
    '''
    #curl --request POST -F "POR=@lat506.por" -F "SPSS=@LAT506.SPS" http://labs.zepheira.com:8880/spss.json
    
    #Useful:
    # * [[http://wiki.math.yorku.ca/index.php/R:_Data_conversion_from_SPSS|R: Data conversion from SPSS]]

    body = StringIO(body)
    form = cgi.FieldStorage(fp=body, environ=WSGI_ENVIRON)
    #for k in form:
    #    print >> sys.stderr, (k, form[k][:100])
    por = form.getvalue('POR')
    assert_not_equal(por, None, msg=POR_REQUIRED)
    spss = form.getvalue('SPSS')
    
    (items, varlabels, valuelabels) = parse_spss(por, spss)

    for count, item in enumerate(items):
        #print >> sys.stderr, row
        item['id'] = item['label'] = '_' + str(count)
        item['type'] = VALUE_SET_TYPE

    return json.dumps({'items': items, VARIABLE_LABELS_TYPE: varlabels, VALUE_LABELS_TYPE: valuelabels}, indent=4)


def parse_spss(spss_por, spss_syntax=None):
    '''
    Uses GNU R to convert SPSS to a simple Python data structure
    Optionally tries to guess long labels from an original .SPS file
    '''
    varlabels = {}
    valuelabels = {}
    if spss_syntax:
        matchinfo = VAR_PAT.search(spss_syntax)
        if matchinfo:
            #print >> sys.stderr, matchinfo.groups
            defns = matchinfo.group(1)
            for defn in VAR_DEF_PAT.finditer(defns):
                varlabels[defn.group(1)] = defn.group(2)

        matchinfo = VALUE_PAT.search(spss_syntax)
        defsets = matchinfo.group(1)
        for defset in VALUE_DEF_SET_PAT.finditer(defsets):
            valuelabelset = {}
            for defn in VALUE_DEF_PAT.finditer(defset.group(2)):
                valuelabelset[defn.group(1)] = defn.group(2)
            valuelabels[defset.group(1)] = valuelabelset

    #print >> sys.stderr, varlabels
    #print >> sys.stderr, valuelabels

    #print >> sys.stderr, por[:100]
    #print >> sys.stderr, spss[:100]
    temp = tempfile.mkstemp(suffix=".por")
    os.write(temp[0], spss_por)

    cmdline = R_FILE_CMD
    process = Popen(cmdline, stdin=PIPE, stdout=PIPE, stderr=PIPE, universal_newlines=True, shell=True)
    
    csvdata, perr = process.communicate(input=R_SCRIPT%temp[1])
    os.close(temp[0])
    os.remove(temp[1])
    if not csvdata:
        print >> sys.stderr, R_SCRIPT%temp[1]
        print >> sys.stderr, perr
        #FIXME: L10N
        raise ValueError('Empty output from the command line.  Probably a failure.  Command line: "%s"'%cmdline)

    def value(k, v):
        if k in valuelabels and v in valuelabels[k]:
            return valuelabels[k][v]
        else:
            return v

    r_reader = csv.DictReader(csvdata.splitlines(), delimiter=';')
    rows = [
        dict(((k, value(k, v.strip())) for (k, v) in row.iteritems()))
        for row in r_reader
    ]

    return (rows, varlabels, valuelabels)




#!/usr/bin/env python

import os
import sys

# Check for the --user option.  This was first added in Python 2.6. 
# For older versions, we'll print out some advice.

if '--user' in sys.argv and sys.version_info[:3] < (2,6,0):
    print """\
************************************************************
The --user option is not supported by this version of Python
(you might consider upgrading to Python 2.6 or newer).  To
install Akara into a user-directory, use the --prefix option to 
setup.py such as this:

    python setup.py install --prefix=$HOME/.local/lib/site-packages

Be sure to change the directory name in the above command to the
actual location where you want the software installed.

To use Akara from a custom install directory, make sure the 
PYTHONPATH environment variable includes the directory before
launching Python. For example:

    env PYTHONPATH=$HOME/.local/lib/site-packages python ...

************************************************************
"""

setuptools_extras = dict(zip_safe=False)

try:

    # ----------------------------------------------------------------------
    # setuptools configuration
    # ----------------------------------------------------------------------
    from setuptools import setup, find_packages

except ImportError:

    # ----------------------------------------------------------------------
    # distutils configuration
    # ----------------------------------------------------------------------

    from distutils.core import setup
    
    # The following functions implement a basic version of the find_packages() 
    # functionality that is missing in distutils.

    # Take a directory name such as '/foo/bar/spam' and split it into all components such as
    # as ('foo','bar','spam').    This is slightly different than os.path.split() which only
    # performs a single split()

    def splitalldirs(dirname):
        base, name = os.path.split(dirname)
        if not base:
            # No more base components to split. Done
            return ()
        else:
            # Recursively split the base directory down to get all of the parts
            return splitalldirs(base) + (name,)

    # Find all packages and return them as a list
    def find_packages():
        packages = []
        for path, dirs, files in os.walk("."):
            packagedirs = []
            for dirname in dirs:
                if os.path.exists(os.path.join(path,dirname,"__init__.py")):
                    components = splitalldirs(path) + (dirname,)
                    packagedirs.append(dirname)
                    packagename = ".".join(components)
                    packages.append(packagename)

            # Only continue with package subdirectories
            dirs[:] = packagedirs
        return packages

# The module files are under "lib/". I want them under "akara/"
def renamed_packages():
    names = []
    for package in find_packages():
        if package.startswith("lib"):
            names.append(package.replace("lib", "akara", 1))
        else:
            raise AssertionError("Unknown package %r" % (package,))
    return names

setuptools_extras["packages"] = renamed_packages()

execfile('lib/version.py')
__version__ = '.'.join(version_info)


setup(name = "Akara",
      version=__version__,
      description='Web components for Amara 2.x',
      author='Uche Ogbuji and others',
      author_email='amara-dev@googlegroups.com',
      url='http://wiki.xml3k.org/Akara',
      package_dir={'akara':'lib'},
      package_data={'akara': ["akara.conf"]},
      scripts =['akara'],
      **setuptools_extras)




# -*- encoding: utf-8 -*-
'''
Demo module which forwards a search term to Google and returns the first hit's URL
'''

import urllib, urllib2
from gettext import gettext as _

# Requires Python 2.6 or http://code.google.com/p/json/
from amara.thirdparty import json

from amara.lib.util import first_item, assert_not_equal
from akara.services import simple_service

Q_REQUIRED = _("The 'q' query parameter is mandatory.")

#text/uri-list from RFC 2483
SERVICE_ID = 'http://purl.org/akara/services/demo/luckygoogle'
@simple_service('GET', SERVICE_ID, 'akara.luckygoogle', 'text/uri-list')
def lucky_google(q=None):
    '''
    A simple and fun transform to return the first hit for a given search
    
    Sample request:
    * curl "http://localhost:8880/akara.luckygoogle?q=zepheira"
    '''
    if q is None:
        raise AssertionError(Q_REQUIRED)
    query = urllib.urlencode({'q' : q})
    url = 'http://ajax.googleapis.com/ajax/services/search/web?v=1.0&' + query
    json_content = json.load(urllib.urlopen(url))
    results = json_content['responseData']['results']
    return results[0]['url'].encode('utf-8') + '\n'



#Useful: http://docs.python.org/library/wsgiref.html
#

import httplib
import sqlite3
from datetime import datetime
from wsgiref.util import shift_path_info, request_uri
from string import Template
from cStringIO import StringIO
from akara.resource import *
from akara.resource.repository import driver
from akara.resource.index import simple_xpath_index

# Templates
wrapper = Template("""\
<html><head><title>$title</title></head><body>
$body
</body></html>
""")

four_oh_four = Template("""\
<html><body>
  <h1>404-ed!</h1>
  The requested URL <i>$url</i> was not found.
</body></html>""")


def alias(environ, start_response):
    '''
    GET - retrieve the resource with the specified alias
    POST - create a resource with the specified alias
    '''
    key = environ['PATH_INFO']
    print 'key', key
    if not key in APPS:
        #404 error
        start_response('404 Not Found', [('content-type', 'text/html')])
        response = four_oh_four.substitute(url=request_uri(environ))
        return [response]
    next = APPS[key]
    return next(environ, start_response)


def response(code):
    return '%i %s'%(code, httplib.responses[code])


def store(environ, start_response):
    dbfile = environ['akara.DBFILE']
    drv = driver(sqlite3.connect(dbfile))
    def head_resource():
        get_resource()
        return ''

    def get_resource():
        key = shift_path_info(environ)

        content1, metadata = drv.get_resource(key)
        if content1 is None:
            #404 error
            start_response('404 Not Found', [('content-type', 'text/html')])
            response = four_oh_four.substitute(url=request_uri(environ))
            return response

        start_response('200 OK', [('content-type', str(metadata[CONTENT_TYPE]))])
        return content1.encode('utf-8')

    def post_resource():
        ctype = environ.get('CONTENT_TYPE', 'application/unknown')
        clen = int(environ.get('CONTENT_LENGTH', None))
        if not clen:
            start_response("411 Length Required", [('Content-Type','text/plain')])
            return ["Length Required"]
        key = shift_path_info(environ)
        now = datetime.now().isoformat()
        md = {
            CREATED: now,
            UPDATED: now,
            CONTENT_LENGTH: clen,
            CONTENT_TYPE: ctype,
        }
        #md = self.standard_index
        content = environ['wsgi.input'].read(clen)
        id = drv.create_resource(content, metadata=md)
        msg = 'Adding %i' % id
        new_uri = str(id)

        headers = [('Content-Type', 'text/plain')]
        headers.append(('Location', new_uri))
        headers.append(('Content-Location', new_uri))

        #environ['akara.etag'] = compute_etag(content)
        headers.append(('Content-Length', str(len(msg))))
        start_response("201 Created", headers)
        
        return msg

    dispatch = {
        'GET': get_resource,
        'HEAD': head_resource,
        'POST': post_resource,
    }

    method = dispatch.get(environ['REQUEST_METHOD'])
    if not method:
        response_headers = [('Content-type','text/plain')]
        start_response(response(httplib.METHOD_NOT_ALLOWED), response_headers)
        return ['Method Not Allowed']
    else:
        return [method()]




"""
WSGI middleware that applies XSLT to an XML response body if needed

See also:
  http://4suite.org/docs/CoreManual.xml#id1964053292
  http://trac.defuze.org/wiki/Picket
  http://projects.dowski.com/view/buffetxslt
  
"""

import re, sys, time
from cStringIO import StringIO
from itertools import *
from xml import sax
from util import geturl

from Ft.Xml import Sax, InputSource, CreateInputSource
from Ft.Xml.Xslt import Processor
from Ft.Lib import Uri

USER_AGENT_REGEXEN = [
  '.*MSIE 5.5.*',
  '.*MSIE 6.0.*',
  '.*MSIE 7.0.*',
  '.*Gecko/2005.*',
  '.*Gecko/2006.*',
  '.*Gecko/2007.*',
  '.*Gecko/2008.*',
  '.*Opera/9.*',
  '.*AppleWebKit/31.*',
  '.*AppleWebKit/4.*',
]

USER_AGENT_REGEXEN = [ re.compile(regex) for regex in USER_AGENT_REGEXEN ]
WSGI_NS = u'http://www.wsgi.org/'

MTYPE_PAT = re.compile('.*/.*xml.*')


class find_xslt_pis(sax.ContentHandler):
    def __init__(self, parser):
        parser.setContentHandler(self)
        self.parser = parser
        return

    def startDocument(self):
        self.ecount = 0
        self.xslt_pi = None

    def startElementNS(self, name, qname, attribs):
        self.ecount += 1
        if self.ecount == 2:
            #We're now within the doc proper, so we're done
            self.parser.setProperty(Sax.PROPERTY_YIELD_RESULT, self.xslt_pi)
        return
            
    def processingInstruction(self, target, data):
        if target == u'xml-stylesheet':
            data = data.split()
            pseudo_attrs = {}
            for d in data:
                seg = d.split('=')
                if len(seg) == 2:
                    pseudo_attrs[seg[0]] = seg[1][1:-1]

            # PI must have both href, type pseudo-attributes;
            # type pseudo-attr must match valid XSLT types;
            # media pseudo-attr must match preferred media
            # (which can be None)
            if (pseudo_attrs.has_key('href')
                and pseudo_attrs.has_key('type')
                and pseudo_attrs['type'] in Processor.XSLT_IMT):
                    self.xslt_pi = pseudo_attrs['href']
                    self.parser.setProperty(Sax.PROPERTY_YIELD_RESULT, self.xslt_pi)
        return

ACTIVE_KEY = 'amara.transform.active'
FORCE_SERVER_SIDE_KEY = 'akara.transform.force-server-side'
CACHEABLE_KEY = 'wsgixml.applyxslt.cacheable'

class applyxslt(object):
    """
    Middleware to conditionally apply XSLT on the server side.
    
    It checks for XSLT transform capability in the client and performs any
    specified transform on the server side if needed.  You can force it to
    run on the server side too by setting
    environ['akara.transform.force-server-side'] to True.
    
    Transforms are specified either through a stylesheet PI or as
    environ['akara.transform.xslt'] which can be a single XSLT input source
    or a list of same.
    
    You can also specify an environ['akara.transform.xsltparams'] environ entry
    with top-level parameters.  Remember that you can use
    amara.xpath.util.parameterize() to prep these
    """
    def __init__(self, app, check_pis=True, use_wsgi_env=True, stock_xslt_params=None,
                 ext_modules=None):
        """
        use_wsgi_env - Optional bool determining whether to make the
            WSGI environment available to the XSLT as top level parameter
            overrides (e.g. wsgi:SCRIPT_NAME and wsgi:wsgi.url_scheme).
            Only passes on values it can handle (UTF-8 strings, Unicode,
            numbers, boolean, lists of "nodes").  Default to True
        stock_xslt_params - optional dict of dicts to also pass along as XSLT
            params.  The outer dict is onf the form: {<namespace>: <inner-dict>}
            And the inner dicts are of the form {pname: pvalue}.  The keys
            (pname) may be given as unicode objects if they have no namespace,
            or as (uri, localname) tuples if they do.  The values are
            (UTF-8 strings, Unicode, numbers, boolean, lists of "nodes").
            This is usually used for passing configuration info into XSLT
        ext_modules - Optional list of modules with XPath and XSLT extensions
        """
        #Set-up phase
        self.wrapped_app = app
        self.use_wsgi_env = use_wsgi_env
        self.stock_xslt_params = stock_xslt_params or {}
        self.ext_modules = ext_modules or []
        self.processor_cache = {}
        self.path_cache = {}
        return

    def __call__(self, environ, start_response):
        #Guess whether the client supports XML+XSLT?
        #See: http://copia.ogbuji.net/blog/2006-08-26/LazyWeb_Ho
        client_ua = environ.get('HTTP_USER_AGENT', '')
        path = environ['PATH_INFO']
        send_browser_xslt = any( ua_pat.match(client_ua) 
                                 for ua_pat in USER_AGENT_REGEXEN )

        #We'll hack a bit for dealing with Python's imperfect nested scopes.
        response_params = []
        def start_response_wrapper(status, response_headers, exc_info=None):
            #Assume response does not use XSLT; do not activate middleware
            environ[ACTIVE_KEY] = False
            #Check for response content type to see whether it is XML
            for name, value in response_headers:
                #content-type value is a media type, defined as
                #media-type = type "/" subtype *( ";" parameter )
                media_type = value.split(';')[0]
                if ( name.lower() == 'content-type'
                     and MTYPE_PAT.match(media_type)):
                     #.endswith('/xml')
                     #      or media_type.find('/xml+') != -1 )):
                    environ[ACTIVE_KEY] = True
            response_params.extend([status, response_headers, exc_info])
            #Replace any write() callable with a dummy that gives an error
            #The idea is to refuse support for apps that use write()
            def dummy_write(data):
                raise RuntimeError('applyxslt does not support the deprecated write() callable in WSGI apps')
            return dummy_write

        #Get the iterator from the application that will yield response
        #body fragments
        iterable = self.wrapped_app(environ, start_response_wrapper)
        (status, response_headers, exc_info) = response_params
        force_server_side = environ.get(FORCE_SERVER_SIDE_KEY, False)
        send_browser_xslt = send_browser_xslt and not force_server_side
        #import pprint; pprint.pprint(environ)

        #This function processes each chunk of output (simple string) from
        #the app, returning The modified chunk to be passed on to the server
        def next_response_block(response_iter):
            if send_browser_xslt or not environ['wsgixml.applyxslt.active']:
                #The client can handle XSLT, or it's not an XML source doc,
                #so nothing for this middleware to do
                start_response(status, response_headers, exc_info)
                for block in response_iter.next():
                    yield block
            elif path in self.path_cache:
                print >> sys.stderr, 'Using cached result for path', path
                imt, content = self.path_cache[path]
                response_headers.append(('content-type', imt))
                start_response(status, response_headers, exc_info)
                yield content
            else:
                yield produce_final_output(''.join(response_iter))
            return  

        #After the app has finished sending its response body fragments
        #if transform is required, it's necessary to send one more chunk,
        #with the fully transformed result
        def produce_final_output(response, response_headers=response_headers):
            if not send_browser_xslt and environ[ACTIVE_KEY]:
                use_pi = False
                if force_server_side and force_server_side != True:
                    #True is a special flag meaning "don't delegate to the browser but still check for XSLT PIs"
                    xslt = force_server_side
                else:
                    #Check for a Stylesheet PI
                    parser = Sax.CreateParser()
                    parser.setFeature(Sax.FEATURE_GENERATOR, True)
                    handler = find_xslt_pis(parser)
                    pi_iter = parser.parse(CreateInputSource(response))
                    try:
                        #Note: only grabs the first PI.  Consider whether we should handle multiple
                        xslt = pi_iter.next()
                    except StopIteration:
                        xslt = None
                    use_pi = True
                if xslt:
                    xslt = xslt.encode('utf-8')
                    result = StringIO()
                    #self.xslt_sources = environ.get(
                    #    'wsgixml.applyxslt.xslt_sources', {})
                    source = InputSource.DefaultFactory.fromString(
                        response, uri=get_request_url(environ))
                    params = {}
                    for ns in self.stock_xslt_params:
                        params.update(setup_xslt_params(ns, self.stock_xslt_params[ns]))
                    start = time.time()
                    if xslt in self.processor_cache:
                        processor = self.processor_cache[xslt]
                        #Any transform would have already been loaded
                        use_pi = False
                        print >> sys.stderr, 'Using cached processor instance for transform', xslt
                        #environ['wsgi.errors'].write('Using cached processor instance for transform %s\n'%xslt)
                    else:
                        print >> sys.stderr, 'Creating new processor instance for transform', xslt
                        #environ['wsgi.errors'].write('Creating new processor instance for transform %s\n'%xslt)
                        processor = Processor.Processor()
                        if self.ext_modules:
                            processor.registerExtensionModules(self.ext_modules)
                        if self.use_wsgi_env:
                            params.update(setup_xslt_params(WSGI_NS, environ))
                        #srcAsUri = OsPathToUri()
                        #if False:
                        if environ.has_key('paste.recursive.include'):
                            #paste's recursive facilities are available, to
                            #so we can get the XSLT with a middleware call
                            #rather than a full Web invocation
                            #print environ['paste.recursive.include']
                            xslt_resp = environ['paste.recursive.include'](xslt)
                            #FIXME: this should be relative to the XSLT, not XML
                            #print xslt_resp, xslt_resp.body
                            isrc = InputSource.DefaultFactory.fromString(
                                xslt_resp.body, get_request_url(environ))
                            processor.appendStylesheet(isrc)
                        else:
                            #We have to make a full Web call to get the XSLT.
                            #4Suite will do that for us in processing the PI
                            if not use_pi:
                                uri = Uri.Absolutize(xslt, get_request_url(environ))
                                isrc = InputSource.DefaultFactory.fromUri(uri)
                                processor.appendStylesheet(isrc)
                        self.processor_cache[xslt] = processor
                    processor.run(source, outputStream=result,
                                  ignorePis=not use_pi, topLevelParams=params)

                    #Strip content-length if present (needs to be
                    #recalculated by server)
                    #Also strip content-type, which will be replaced below
                    response_headers = [ (name, value)
                        for name, value in response_headers
                            if ( name.lower()
                                 not in ['content-length', 'content-type'])
                    ]
                    #Put in the updated content type
                    imt = processor.outputParams.mediaType
                    content = result.getvalue()
                    if environ.get(CACHEABLE_KEY):
                        self.path_cache[path] = imt, content
                    response_headers.append(('content-type', imt))
                    start_response(status, response_headers, exc_info)
                    end = time.time()
                    print >> sys.stderr, '%s: elapsed time: %0.3f\n'%(xslt, end-start)
                    #environ['wsgi.errors'].write('%s: elapsed time: %0.3f\n'%(xslt, end-start))
                    return content
                    
            #If it reaches this point, no XSLT was applied.
            return

        return iterwrapper(iterable, next_response_block)


#if __name__ == '__main__':
    



# -*- encoding: utf-8 -*-
'''
@ 2011 by Uche ogbuji <uche@ogbuji.net>

This file is part of the open source Akara project,
provided under the Apache 2.0 license.
See the files LICENSE and NOTICE for details.
Project home, documentation, distributions: http://wiki.xml3k.org/Akara

 Module name:: sysinfo
 
Responds to get with debug info about the system

= Defined REST entry points =

http://purl.org/akara/services/demo/sysinfo (akara.sysinfo) Handles GET

= Configuration =

No configuration required, but any config is reflected in the return messages

= Notes on security =

This module can be a huge security risk.  Please do not deploy it on any sensitive server beyond a debug period.
'''

import amara
from akara import global_config, request
from akara.services import simple_service
import pprint

SYSINFO_SERVICE_ID = 'http://purl.org/xml3k/akara/services/demo/sysinfo'


@simple_service('GET', SYSINFO_SERVICE_ID, 'akara.sysinfo', 'text/plain')
def akara_sysinfo():
    '''
    Sample request:
    curl "http://localhost:8880/akara.sysinfo"
    '''
    gdict = dict( (k, v) for (k, v) in global_config.__dict__.items() if k not in ['__builtins__', '__file__', '__name__', '__package__', '__doc__'])
    body = 'environ'
    body += '\n'
    body += pprint.pformat(request.environ)
    body += '\n\n'
    body += 'global_config'
    body += '\n'
    body += pprint.pformat(gdict)
    body += '\n'
    #pprint.pformat(request.environ)
    return body




# -*- coding: iso-8859-1 -*-
# 
"""
@ 2009 by Uche ogbuji <uche@ogbuji.net>

This file is part of the open source Akara project,
provided under the Apache 2.0 license.
See the files LICENSE and NOTICE for details.
Project home, documentation, distributions: http://wiki.xml3k.org/Akara

 Module name:: collection

= Defined REST entry points =

http://purl.org/akara/services/demo/collection (akara.collection) Handles HEAD, GET, POST, PUT

= Configuration =

You'll need a config entry such as:

[collection]
folder = /tmp/collection

= Notes on security =

This module is a very simple demo, and does not pay much attention to security at all.
The directory you specify in the config will be exposed to the Web, and thus to attack,
although measures are taken to prevent obvious hierarchical manipulation to access
outside that folder

Use with great care.

"""
#Detailed license and copyright information: http://4suite.org/COPYRIGHT

from __future__ import with_statement

SAMPLE_QUERIES_DOC = '''
= Some sample queries =

Get a list of files:

    curl http://localhost:8880/collection

Add a file:

    curl --request POST --data-binary "@foo.txt" --header "Content-Type: text/plain" "http://localhost:8880/collection"
    Note: You might want to see the response headers.  Add "-i" right after "curl" (works with any of these commands)

Get a file:

    curl http://localhost:8880/collection/1

Update file content

    curl --request PUT --data-binary "@foo1.txt" --header "Content-Type: text/plain" "http://localhost:8880/collection/1"
'''

__doc__ += SAMPLE_QUERIES_DOC

import sys
import os
import time
import httplib
from string import Template
from gettext import gettext as _
from contextlib import closing
from wsgiref.util import shift_path_info, request_uri

from akara.util import multipart_post_handler, wsgibase, http_method_handler

from akara.services import method_dispatcher
from akara import response

#AKARA is automatically defined at global scope for a module running within Akara
BASE = None
def akara_init(config):
    global BASE
    BASE = config["collection"].BASE

# Templates
four_oh_four = Template("""
<html><body>
  <h1>404-ed!</h1>
  The requested URL <i>$fronturl</i> was not found (<i>$backurl</i> in the target wiki).
</body></html>""")

SERVICE_ID = 'http://purl.org/akara/services/demo/collection'
DEFAULT_MOUNT = 'akara.collection'

def status_response(code):
    return '%i %s'%(code, httplib.responses[code])

#
@method_dispatcher(SERVICE_ID, DEFAULT_MOUNT)
def collection_resource():
    __doc__ = SAMPLE_QUERIES_DOC
    return


@collection_resource.method("GET")
def get_file(environ, start_response):
    '''
    GETting the collection resource itself returns a simple file listing.
    
    GETting a subsidiary resource returns the file
    '''
    print >> sys.stderr, 'GRIPPO', environ['PATH_INFO']
    if environ['PATH_INFO'] == '/':
        #Get index
        start_response(status_response(httplib.OK), [("Content-Type", "text/plain")])
        return '\n'.join(os.listdir(BASE)) + '\n'
    resource_fname = shift_path_info(environ)
    #Not needed because the shift_path_info will ignore anything after the '/' and they'll probably get a 404
    #'..' will not be expanded by os.path.join
    #if "/" in resource_fname:
    #    start_response(status_response(httplib.BAD_REQUEST), [("Content-Type", "text/plain")])
    #    return 'You must not include forward slashes in your request (%s)'%resource_fname
    resource_path = os.path.join(BASE, resource_fname)
    print >> sys.stderr, 'Getting the file at: ', resource_fname
    try:
        f = open(resource_path, 'rb')
        #FIXME: do it chunk by chunk
        rbody = f.read()
        #FIXME: work out content type mappings (perhaps by file extension)
        start_response(status_response(httplib.OK), [("Content-Type", "text/plain")])
        return rbody
    except IOError:
        rbody = four_oh_four.substitute(fronturl=request_uri(environ), backurl=resource_fname)
        start_response(status_response(httplib.NOT_FOUND), [("Content-Type", "text/html")])
        return rbody

#
@collection_resource.method("POST")
def post_file(environ, start_response):
    '''
    Add a new file to the collection
    '''
    #Not needed because the shift_path_info will ignore anything after the '/' and they'll probably get a 404
    #'..' will not be expanded by os.path.join
    #if "/" in resource_fname:
    #    start_response(status_response(httplib.BAD_REQUEST), [("Content-Type", "text/plain")])
    #    return 'You must not include forward slashes in your request (%s)'%resource_fname
    fname = str(int(time.time()))
    #resource_fname = shift_path_info(environ)
    resource_path = os.path.join(BASE, fname)
    fp = open(resource_path, 'wb')

    if not read_http_body_to_file(environ, start_response, fp):
        return 'Content length Required'

    msg = 'File created OK: %s\n'%(fname)
    print >> sys.stderr, 'Creating a file at: ', resource_path

    #response.add_header("Content-Length", str(len(msg)))
    #FIXME: use the full URI for Location header
    start_response(status_response(httplib.CREATED), [("Content-Type", "text/plain"), ("Content-Location", fname)])
    return msg


#def check_auth(self, user=None, password=None):
@collection_resource.method("PUT")
def put_page(environ, start_response):
    '''
    '''
    raise NotImplementedErr


#
CHUNKLEN = 4096
def read_http_body_to_file(environ, start_response, fp):
    '''
    Handle the reading of a file from an HTTP message body (file pointer from wsgi.input)
    in chunks to a temporary file
    Returns the file path of the resulting temp file
    '''
    clen = int(environ.get('CONTENT_LENGTH', None))
    if not clen:
        start_response(status_response(httplib.LENGTH_REQUIRED), [("Content-Type", "text/plain")])
        return False
    http_body = environ['wsgi.input']
    while clen != 0:
        chunk_len = min(CHUNKLEN, clen)
        data = http_body.read(chunk_len)
        if data:
            fp.write(data)
            clen -= chunk_len
        else:
            clen = 0
    fp.close()
    return True





# -*- coding: iso-8859-1 -*-
# 
# akara
# © 2008-2010 by Uche Ogbuji and Zepheira LLC
#
"""
akara - top-level module for an Akara installation

Copyright 2009-2010 Uche Ogbuji and Zepheira LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Public submodules are:
  registry - direct interface to the resource handler registry
  services - decorators to simplify common ways of writing
       and registring resource handlers

     These modules contain data which change for each HTTP request.
     They are used by some of the services decorators.
  request - information about an incoming HTTP query
  response - data for the outgoing HTTP response

"""

import sys

# Initializes logging and makes the logger public
from akara.logger_config import _logger as logger

# The contents of the "akara.conf" module, as a global namespace dict
raw_config = None

# Access to the per-module configuration. Used like:
#  akara.module_config("akara.demo.xslt")  -- full path name
#  akara.module_config("xslt")   -- in most cases, the last term works fine
#  akara.module_config(__name__)    -- easy way for extension modules to know its own name
#  akara.module_config()    -- get the __name__ automatically

def module_config(path = None):
    if path is None:
        # Let people call this without specifying a name
        path = sys._getframe(1).f_globals["__name__"]

    # The path can either be a full path, like "akara.demo.xslt"
    # or a short name like "xslt". Look for the full name first.
    # We use classes for the config information, which means
    # it isn't so simple, as the class doesn't have a full name.
    # Here's how to specify the full name, to match the path exactly:
    # 
    #    class xslt:
    #        akara_name = "akara.demo.xslt"
    #
    for name, obj in raw_config.items():
        if name[:1] == "_":
            continue
        if hasattr(obj, "akara_name") and obj.akara_name == path:
            return ModuleConfig(path, obj)

    # There's also a shorthand notation, which is useful for
    # the (expected to be) common case where there's no conflict;
    # use the last term of the path as the name of the class.
    class_name = path.rsplit(".", 1)[-1]
    try:
        klass = raw_config[class_name]
    except KeyError:
        # An extension should not require a configuration.
        # Testing for configuration adds if (or try/except) tests, which complicates code.
        # The purpose of returning a NoModuleConfig is to let people write code like:
        #   arg = module_config().get("order", "spam and eggs")
        # and have code work as expected.
        return NoModuleConfig(path)
    else:
        if hasattr(klass, "akara_name"):
            path = klass.akara_name
        return ModuleConfig(path, klass)


# Wrap the configuration object so the class attributes can be
# accessed as dictionaries via [] and get. Why? I think it simplifies
# parameter processing to use config.get("name", default) rather than
# getattr(config, "name", default) and because I think this will grow
# terms like "get_string()" to help with type validation.

class ModuleConfig(object):
    def __init__(self, path, config_class):
        self.path = path
        self.config_class = config_class
        
    def __getitem__(self, name):
        try:
            return getattr(self.config_class, name)
        except AttributeError:
            raise KeyError(name)
        
    def get(self, name, default=None):
        return getattr(self.config_class, name, default)

    def require(self, name, what=None):
        try:
            return getattr(self.config_class, name)
        except AttributeError:
            pass

        msg = (
            "Akara configuration section %(path)r is missing the required parameter %(name)r"
            % dict(path=self.path, name=name) )
        
        if what is not None:
            msg += ": " + what
        raise AttributeError(msg)

    def warn(self, name, default, what=None):
        try:
            return getattr(self.config_class, name)
        except AttributeError:
            pass
        
        msg = "Akara configuration section %(path)r should have the parameter %(name)r"
        if what is not None:
            msg += " (%(what)s)"
        msg += ". Using %(default)r instead."
        msg = msg % dict(path=self.path, name=name, default=default, what=what)
        logger.warn(msg)

        return default


# Used when there is no configuration section for the module.

class NoModuleConfig(object):
    def __init__(self, path):
        self.path = path
        
    def __getitem__(self, name):
        raise KeyError(name)
    
    def get(self, name, default=None):
        return default
    
    def __nonzero__(self):
        # Can be used to test if there was a configuration section, as in
        #    if not module_config("Akara"): print "Something is wrong!"
        return False
    
    def require(self, name, what=None):
        msg = ("Akara configuration section %(path)r is missing. It must have the "
               "parameter %(name)r") % dict(path=self.path, name=name)
        if what is not None:
            msg += ": " + what
        raise AttributeError(msg)

    def warn(self, name, default, what=None):
        msg = ("Akara configuration section %(path)r is missing. "
               "It should have the parameter %(name)r")
        if what is not None:
            msg += " (%(what)s)"
        msg += ". Using %(default)r instead."
        msg = msg % dict(path=self.path, name=name, default=default, what=what)
        logger.warn(msg)

        return default


from version import version_info
__version__ = '.'.join(version_info)



"""Information for the outgoing response

  code - the HTTP response code (default is "200 Ok")
  headers - a list of key/value pairs used for the WSGI start_response

"""

code = None
headers = []

def add_header(key, value):
    """Helper function to append (key, value) to the list of response headers"""
    headers.append( (key, value) )

# Eventually add cookie support?
